from datetime import date

from twisted.internet.defer import inlineCallbacks

from vumi.application.tests.helpers import ApplicationHelper
from vumi.demos.static_reply import StaticReplyApplication
from vumi.tests.helpers import VumiTestCase


class TestStaticReplyApplication(VumiTestCase):
    def setUp(self):
        self.app_helper = self.add_helper(
            ApplicationHelper(StaticReplyApplication))

    @inlineCallbacks
    def test_receive_message_custom(self):
        yield self.app_helper.get_application({
            'reply_text': 'Hi {user}',
        })
        yield self.app_helper.make_dispatch_inbound(
            "Hello", from_addr='from_addr')
        [reply] = self.app_helper.get_dispatched_outbound()
        self.assertEqual('Hi from_addr', reply['content'])
        self.assertEqual(u'close', reply['session_event'])

    @inlineCallbacks
    def test_receive_message(self):
        yield self.app_helper.get_application({})
        yield self.app_helper.make_dispatch_inbound(
            "Hello", from_addr='from_addr')
        [reply] = self.app_helper.get_dispatched_outbound()
        self.assertEqual('Hello from_addr at %s.' % (date.today(),),
                         reply['content'])
        self.assertEqual(u'close', reply['session_event'])



import os.path

from twisted.internet import reactor
from twisted.internet.defer import Deferred, inlineCallbacks
from twisted.web.server import Site, NOT_DONE_YET
from twisted.web.resource import Resource
from twisted.web import http
from twisted.web.client import WebClientContextFactory, Agent
from twisted.internet.protocol import Protocol, Factory

from vumi.utils import (normalize_msisdn, vumi_resource_path, cleanup_msisdn,
                        get_operator_name, http_request, http_request_full,
                        get_first_word, redis_from_config, build_web_site,
                        LogFilterSite, PkgResources)
from vumi.persist.fake_redis import FakeRedis
from vumi.tests.helpers import VumiTestCase, import_skip


class DummyRequest(object):
    def __init__(self, postpath, prepath):
        self.postpath = postpath
        self.prepath = prepath


class TestNormalizeMsisdn(VumiTestCase):
    def test_leading_zero(self):
        self.assertEqual(normalize_msisdn('0761234567', '27'),
                         '+27761234567')

    def test_double_leading_zero(self):
        self.assertEqual(normalize_msisdn('0027761234567', '27'),
                         '+27761234567')

    def test_leading_plus(self):
        self.assertEqual(normalize_msisdn('+27761234567', '27'),
                         '+27761234567')

    def test_no_leading_plus_or_zero(self):
        self.assertEqual(normalize_msisdn('27761234567', '27'),
                         '+27761234567')

    def test_short_address(self):
        self.assertEqual(normalize_msisdn('1234'), '1234')
        self.assertEqual(normalize_msisdn('12345'), '12345')

    def test_short_address_with_leading_plus(self):
        self.assertEqual(normalize_msisdn('+12345'), '+12345')

    def test_unicode_addr_remains_unicode(self):
        addr = normalize_msisdn(u'0761234567', '27')
        self.assertEqual(addr, u'+27761234567')
        self.assertTrue(isinstance(addr, unicode))

    def test_str_addr_remains_str(self):
        addr = normalize_msisdn('0761234567', '27')
        self.assertEqual(addr, '+27761234567')
        self.assertTrue(isinstance(addr, str))


class TestUtils(VumiTestCase):
    def test_make_campaign_path_abs(self):
        vumi_tests_path = os.path.dirname(__file__)
        vumi_path = os.path.dirname(os.path.dirname(vumi_tests_path))
        self.assertEqual('/foo/bar', vumi_resource_path('/foo/bar'))
        self.assertEqual(os.path.join(vumi_path, 'vumi/resources/foo/bar'),
                         vumi_resource_path('foo/bar'))

    def test_cleanup_msisdn(self):
        self.assertEqual('27761234567', cleanup_msisdn('27761234567', '27'))
        self.assertEqual('27761234567', cleanup_msisdn('+27761234567', '27'))
        self.assertEqual('27761234567', cleanup_msisdn('0761234567', '27'))

    def test_get_operator_name(self):
        mapping = {'27': {'2782': 'VODACOM', '2783': 'MTN'}}
        self.assertEqual('MTN', get_operator_name('27831234567', mapping))
        self.assertEqual('VODACOM', get_operator_name('27821234567', mapping))
        self.assertEqual('UNKNOWN', get_operator_name('27801234567', mapping))

    def test_get_first_word(self):
        self.assertEqual('KEYWORD',
                         get_first_word('KEYWORD rest of the message'))
        self.assertEqual('', get_first_word(''))
        self.assertEqual('', get_first_word(None))

    def test_redis_from_config_str(self):
        try:
            fake_redis = redis_from_config("FAKE_REDIS")
        except ImportError, e:
            import_skip(e, 'redis')
        self.assertTrue(isinstance(fake_redis, FakeRedis))

    def test_redis_from_config_fake_redis(self):
        fake_redis = FakeRedis()
        try:
            self.assertEqual(redis_from_config(fake_redis), fake_redis)
        except ImportError, e:
            import_skip(e, 'redis')

    def get_resource(self, path, site):
        request = DummyRequest(postpath=path.split('/'), prepath=[])
        return site.getResourceFor(request)

    def test_build_web_site(self):
        resource_a = Resource()
        resource_b = Resource()
        site = build_web_site({
            'foo/a': resource_a,
            'bar/b': resource_b,
        })
        self.assertEqual(self.get_resource('foo/a', site), resource_a)
        self.assertEqual(self.get_resource('bar/b', site), resource_b)
        self.assertTrue(isinstance(site, LogFilterSite))

    def test_build_web_site_with_overlapping_paths(self):
        resource_a = Resource()
        resource_b = Resource()
        site = build_web_site({
            'foo/a': resource_a,
            'foo/b': resource_b,
        })
        self.assertEqual(self.get_resource('foo/a', site), resource_a)
        self.assertEqual(self.get_resource('foo/b', site), resource_b)
        self.assertTrue(isinstance(site, LogFilterSite))

    def test_build_web_site_with_custom_site_class(self):
        site = build_web_site({}, site_class=Site)
        self.assertTrue(isinstance(site, Site))
        self.assertFalse(isinstance(site, LogFilterSite))


class FakeHTTP10(Protocol):
    def dataReceived(self, data):
        self.transport.write(self.factory.response_body)
        self.transport.loseConnection()


class TestHttpUtils(VumiTestCase):

    class InterruptHttp(Exception):
        """Indicates that test server should halt http reply"""
        pass

    @inlineCallbacks
    def setUp(self):
        self.root = Resource()
        self.root.isLeaf = True
        site_factory = Site(self.root)
        self.webserver = yield reactor.listenTCP(
            0, site_factory, interface='127.0.0.1')
        # This is a lambda because we replace self.webserver in a test.
        self.add_cleanup(lambda: self.webserver.loseConnection())
        addr = self.webserver.getHost()
        self.url = "http://%s:%s/" % (addr.host, addr.port)

    def set_render(self, f, d=None):
        def render(request):
            request.setHeader('Content-Type', 'text/plain')
            try:
                data = f(request)
                request.setResponseCode(http.OK)
            except self.InterruptHttp:
                reactor.callLater(0, d.callback, request)
                return NOT_DONE_YET
            except Exception, err:
                data = str(err)
                request.setResponseCode(http.INTERNAL_SERVER_ERROR)
            return data

        self.root.render = render

    @inlineCallbacks
    def test_http_request_ok(self):
        self.set_render(lambda r: "Yay")
        data = yield http_request(self.url, '')
        self.assertEqual(data, "Yay")

    @inlineCallbacks
    def test_http_request_err(self):
        def err(r):
            raise ValueError("Bad")
        self.set_render(err)
        data = yield http_request(self.url, '')
        self.assertEqual(data, "Bad")

    @inlineCallbacks
    def test_http_request_with_custom_context_factory(self):
        self.set_render(lambda r: "Yay")

        ctxt = WebClientContextFactory()

        class FakeAgent(Agent):
            def __init__(slf, reactor, contextFactory=None):
                self.assertEqual(contextFactory, ctxt)
                super(FakeAgent, slf).__init__(reactor, contextFactory)

        request = yield http_request_full(self.url, '',
                                          context_factory=ctxt,
                                          agent_class=FakeAgent)
        self.assertEqual(request.delivered_body, "Yay")
        self.assertEqual(request.code, http.OK)

    @inlineCallbacks
    def test_http_request_full_drop(self):
        def interrupt(r):
            raise self.InterruptHttp()
        got_request = Deferred()
        self.set_render(interrupt, got_request)

        got_data = http_request_full(self.url, '')

        request = yield got_request
        request.setResponseCode(http.OK)
        request.write("Foo!")
        request.transport.loseConnection()

        def callback(reason):
            self.assertTrue(
                reason.check("twisted.web._newclient.ResponseFailed"))
            done.callback(None)
        done = Deferred()

        got_data.addBoth(callback)

        yield done

    @inlineCallbacks
    def test_http_request_full_ok(self):
        self.set_render(lambda r: "Yay")
        request = yield http_request_full(self.url, '')
        self.assertEqual(request.delivered_body, "Yay")
        self.assertEqual(request.code, http.OK)

    @inlineCallbacks
    def test_http_request_full_headers(self):
        def check_ua(request):
            self.assertEqual('blah', request.getHeader('user-agent'))
            return "Yay"
        self.set_render(check_ua)

        request = yield http_request_full(self.url, '',
                                          {'User-Agent': ['blah']})
        self.assertEqual(request.delivered_body, "Yay")
        self.assertEqual(request.code, http.OK)

        request = yield http_request_full(self.url, '', {'User-Agent': 'blah'})
        self.assertEqual(request.delivered_body, "Yay")
        self.assertEqual(request.code, http.OK)

    @inlineCallbacks
    def test_http_request_full_err(self):
        def err(r):
            raise ValueError("Bad")
        self.set_render(err)
        request = yield http_request_full(self.url, '')
        self.assertEqual(request.delivered_body, "Bad")
        self.assertEqual(request.code, http.INTERNAL_SERVER_ERROR)

    @inlineCallbacks
    def test_http_request_potential_data_loss(self):
        self.webserver.loseConnection()
        factory = Factory()
        factory.protocol = FakeHTTP10
        factory.response_body = (
            "HTTP/1.0 201 CREATED\r\n"
            "Date: Mon, 23 Jan 2012 15:08:47 GMT\r\n"
            "Server: Fake HTTP 1.0\r\n"
            "Content-Type: text/html; charset=utf-8\r\n"
            "\r\n"
            "Yay")
        self.webserver = yield reactor.listenTCP(
            0, factory, interface='127.0.0.1')
        addr = self.webserver.getHost()
        self.url = "http://%s:%s/" % (addr.host, addr.port)

        data = yield http_request(self.url, '')
        self.assertEqual(data, "Yay")

    @inlineCallbacks
    def test_http_request_full_data_limit(self):
        self.set_render(lambda r: "Four")

        d = http_request_full(self.url, '', data_limit=3)

        def check_response(reason):
            self.assertTrue(reason.check('vumi.utils.HttpDataLimitError'))
            self.assertEqual(reason.getErrorMessage(),
                             "More than 3 bytes received")

        d.addBoth(check_response)
        yield d

    @inlineCallbacks
    def test_http_request_full_ok_with_timeout_set(self):
        # If we don't cancel the pending timeout check this test will fail with
        # a dirty reactor.
        self.set_render(lambda r: "Yay")
        request = yield http_request_full(self.url, '', timeout=100)
        self.assertEqual(request.delivered_body, "Yay")
        self.assertEqual(request.code, http.OK)

    @inlineCallbacks
    def test_http_request_full_timeout_before_connect(self):
        # This tests the case where the client times out before
        # successfully connecting to the server.

        # don't need to call .set_render because the request
        # will never make it to the server
        d = http_request_full(self.url, '', timeout=0)

        def check_response(reason):
            self.assertTrue(reason.check('vumi.utils.HttpTimeoutError'))

        d.addBoth(check_response)
        yield d

    @inlineCallbacks
    def test_http_request_full_timeout_after_connect(self):
        # This tests the case where the client connects but then
        # times out before the server sends any data.

        def interrupt(r):
            raise self.InterruptHttp
        request_started = Deferred()
        self.set_render(interrupt, request_started)

        client_done = http_request_full(self.url, '', timeout=0.1)

        def check_response(reason):
            self.assertTrue(reason.check('vumi.utils.HttpTimeoutError'))

        client_done.addBoth(check_response)
        yield client_done

        request = yield request_started
        request.transport.loseConnection()

    @inlineCallbacks
    def test_http_request_full_timeout_after_first_receive(self):
        # This tests the case where the client connects, receives
        # some data and creates its receiver but then times out
        # because the server takes too long to finish sending the data.

        def interrupt(r):
            raise self.InterruptHttp
        request_started = Deferred()
        self.set_render(interrupt, request_started)

        client_done = http_request_full(self.url, '', timeout=0.1)

        request = yield request_started
        request.write("some data")

        def check_server_response(reason):
            self.assertTrue(reason.check('twisted.internet.error'
                                         '.ConnectionDone'))

        request_done = request.notifyFinish()
        request_done.addBoth(check_server_response)
        yield request_done

        def check_client_response(reason):
            self.assertTrue(reason.check('vumi.utils.HttpTimeoutError'))
        client_done.addBoth(check_client_response)
        yield client_done


class TestPkgResources(VumiTestCase):

    vumi_tests_path = os.path.dirname(__file__)

    def test_absolute_path(self):
        pkg = PkgResources("vumi.tests")
        self.assertEqual('/foo/bar', pkg.path('/foo/bar'))

    def test_relative_path(self):
        pkg = PkgResources("vumi.tests")
        self.assertEqual(os.path.join(self.vumi_tests_path, 'foo/bar'),
                         pkg.path('foo/bar'))



import json
from collections import namedtuple

from twisted.internet.defer import inlineCallbacks, Deferred

from vumi.message import Message
from vumi.service import Worker, WorkerCreator
from vumi.tests.helpers import VumiTestCase, WorkerHelper


def fake_amq_message(dictionary, delivery_tag='delivery_tag'):
    Content = namedtuple('Content', ['body'])
    AMQMessage = namedtuple('Message', ['content', 'delivery_tag'])
    return AMQMessage(delivery_tag=delivery_tag,
                      content=Content(body=json.dumps(dictionary)))


class TestService(VumiTestCase):
    def setUp(self):
        self.worker_helper = self.add_helper(WorkerHelper())

    @inlineCallbacks
    def test_consume(self):
        """The consume helper should direct all incoming messages matching the
        specified routing_key, queue_name & exchange to the given callback"""

        message = fake_amq_message({"key": "value"})
        worker = yield self.worker_helper.get_worker(Worker, {}, start=False)

        # buffer to check messages consumed
        log = []
        # consume all messages on the given routing key and append
        # them to the log
        yield worker.consume('test.routing.key', lambda msg: log.append(msg))
        # if all works well then the consume method should funnel the test
        # message straight to the callback, the callback will apend it to the
        # log and we can test it.
        self.worker_helper.broker.basic_publish('vumi', 'test.routing.key',
                                                message.content)
        yield self.worker_helper.broker.wait_delivery()
        self.assertEquals(log, [Message(key="value")])

    @inlineCallbacks
    def test_consume_with_prefetch(self):
        """The consume helper should direct all incoming messages matching the
        specified routing_key, queue_name & exchange to the given callback"""

        worker = yield self.worker_helper.get_worker(Worker, {}, start=False)

        # buffer to check messages consumed
        log = []
        # consume all messages on the given routing key and append
        # them to the log
        consumer = yield worker.consume(
            'test.routing.key', lambda msg: log.append(msg), prefetch_count=10)
        self.assertEqual(10, consumer.channel._get_consumer_prefetch(
            consumer._consumer_tag))

    @inlineCallbacks
    def test_broken_consume(self):
        """
        If a consumer function throws an exception, we still decrement the
        progress counter.
        """
        worker = yield self.worker_helper.get_worker(Worker, {}, start=False)
        start_d = Deferred()
        pause_d = Deferred()

        @inlineCallbacks
        def consume_func(msg):
            start_d.callback(None)
            yield pause_d
            raise Exception("oops")

        consumer = yield worker.consume('test.routing.key', consume_func)
        message = fake_amq_message({"key": "value"})

        # Assert we have no messages being processed and publish a message.
        self.assertEqual(consumer._in_progress, 0)
        self.worker_helper.broker.basic_publish(
            'vumi', 'test.routing.key', message.content)
        # Wait for the processing to start and assert that it's in progress.
        yield start_d
        self.assertEqual(consumer._in_progress, 1)
        # Unpause the processing and wait for it to finish.
        pause_d.callback(None)
        yield self.worker_helper.kick_delivery()
        # If we get here without timing out, everything should be happy.
        self.assertEqual(consumer._in_progress, 0)
        [failure] = self.flushLoggedErrors()
        self.assertEqual(failure.getErrorMessage(), "oops")

    @inlineCallbacks
    def test_start_publisher(self):
        """The publisher should publish"""
        worker = WorkerHelper.get_worker_raw(Worker, {})
        publisher = yield worker.publish_to('test.routing.key')
        self.assertEquals(publisher.routing_key, 'test.routing.key')
        publisher.publish_message(Message(key="value"))
        [published_msg] = publisher.channel.broker.get_dispatched(
            'vumi', 'test.routing.key')

        self.assertEquals(published_msg.body, '{"key": "value"}')
        self.assertEquals(published_msg.properties, {'delivery mode': 2})


class LoadableTestWorker(Worker):
    def poke(self):
        return "poke"


class NoQueueWorkerCreator(WorkerCreator):
    def _connect(self, *_args, **_kw):
        pass


class TestWorkerCreator(VumiTestCase):
    def get_creator(self, **options):
        vumi_options = {
            "hostname": "127.0.0.1",
            "port": 5672,
            "username": "vumitest",
            "password": "vumitest",
            "vhost": "/test",
            "specfile": "amqp-spec-0-8.xml",
            }
        vumi_options.update(options)
        return NoQueueWorkerCreator(vumi_options)

    def test_create_worker(self):
        """
        WorkerCreator should successfully create a test worker instance.
        """

        creator = self.get_creator()
        worker_class = "%s.%s" % (LoadableTestWorker.__module__,
                                  LoadableTestWorker.__name__)
        worker = creator.create_worker(worker_class, {})
        self.assertEquals("poke", worker.poke())



#!/usr/bin/env python
# -*- test-case-name: vumi.scripts.tests.test_vumi_count_models -*-

import re
import sys

from twisted.internet.defer import inlineCallbacks, succeed
from twisted.internet.task import react
from twisted.python import usage

from vumi.utils import load_class_by_string
from vumi.persist.txriak_manager import TxRiakManager


class Options(usage.Options):
    optParameters = [
        ["model", "m", None,
         "Full Python name of the model class to count."
         " E.g. 'vumi.components.message_store.InboundMessage'."],
        ["bucket-prefix", "b", None,
         "The bucket prefix for the Riak manager."],
        ["index-field", None, None,
         "Field with index to query. If omitted, all keys in the bucket will"
         " be counted and no `index-value-*' parameters are allowed."],
        ["index-value", None, None, "Exact match value or start of range."],
        ["index-value-end", None, None,
         "End of range. If ommitted, an exact match query will be used."],
        ["index-value-regex", None, None, "Regex to filter index values."],
        ["index-page-size", None, "1000",
         "The number of keys to fetch in each index query."],
    ]

    longdesc = """
    Index-based model counter. This makes paginated index queries, optionally
    filters the results by applying a regex to the index value, and returns a
    count of all matching models.
    """

    def ensure_dependent_option(self, needed, needs):
        """
        Raise UsageError if `needs` is provided without `needed`.
        """
        if self[needed] is None and self[needs] is not None:
            raise usage.UsageError("%s requires %s to be specified." % (
                needs, needed))

    def postOptions(self):
        if self["model"] is None:
            raise usage.UsageError("Please specify a model class.")
        if self["bucket-prefix"] is None:
            raise usage.UsageError("Please specify a bucket prefix.")
        self.ensure_dependent_option("index-field", "index-value")
        self.ensure_dependent_option("index-value", "index-field")
        self.ensure_dependent_option("index-value", "index-value-end")
        self.ensure_dependent_option("index-value-end", "index-value-regex")
        self["index-page-size"] = int(self['index-page-size'])


class ProgressEmitter(object):
    """Report progress as the number of items processed to an emitter."""

    def __init__(self, emit, batch_size):
        self.emit = emit
        self.batch_size = batch_size
        self.processed = 0

    def update(self, value):
        if (value / self.batch_size) > (self.processed / self.batch_size):
            self.emit(value)
        self.processed = value


class ModelCounter(object):
    def __init__(self, options):
        self.options = options
        model_cls = load_class_by_string(options['model'])
        riak_config = {
            'bucket_prefix': options['bucket-prefix'],
        }
        manager = self.get_riak_manager(riak_config)
        self.model = manager.proxy(model_cls)

    def get_riak_manager(self, riak_config):
        return TxRiakManager.from_config(riak_config)

    def emit(self, s):
        print s

    def count_keys(self, keys, filter_regex):
        """
        Count keys in an index page, filtering by regex if necessary.
        """
        if filter_regex is not None:
            keys = [(v, k) for v, k in keys if filter_regex.match(v)]
        return len(keys)

    @inlineCallbacks
    def count_pages(self, index_page, filter_regex):
        emit_progress = lambda t: self.emit(
            "%s object%s counted." % (t, "" if t == 1 else "s"))
        progress = ProgressEmitter(
            emit_progress, self.options["index-page-size"])
        counted = 0
        while index_page is not None:
            if index_page.has_next_page():
                next_page_d = index_page.next_page()
            else:
                next_page_d = succeed(None)
            counted += self.count_keys(list(index_page), filter_regex)
            progress.update(counted)
            index_page = yield next_page_d
        self.emit("Done, %s object%s found." % (
            counted, "" if counted == 1 else "s"))

    @inlineCallbacks
    def count_all_keys(self):
        """
        Perform an index query to get all keys and count them.
        """
        self.emit("Counting all keys ...")
        index_page = yield self.model.all_keys_page(
            max_results=self.options["index-page-size"])
        yield self.count_pages(index_page, filter_regex=None)

    @inlineCallbacks
    def count_index_keys(self):
        """
        Perform an index query to get all matching keys and count them.
        """
        filter_regex = self.options["index-value-regex"]
        if filter_regex is not None:
            filter_regex = re.compile(filter_regex)
        self.emit("Counting ...")
        index_page = yield self.model.index_keys_page(
            field_name=self.options["index-field"],
            value=self.options["index-value"],
            end_value=self.options["index-value-end"],
            max_results=self.options["index-page-size"],
            return_terms=True)
        yield self.count_pages(index_page, filter_regex=filter_regex)

    def run(self):
        if self.options["index-field"] is None:
            return self.count_all_keys()
        else:
            return self.count_index_keys()


def main(_reactor, name, *args):
    try:
        options = Options()
        options.parseOptions(args)
    except usage.UsageError, errortext:
        print '%s: %s' % (name, errortext)
        print '%s: Try --help for usage details.' % (name,)
        sys.exit(1)

    model_counter = ModelCounter(options)
    return model_counter.run()


if __name__ == '__main__':
    react(main, sys.argv)



from twisted.internet.defer import inlineCallbacks
from twisted.internet.task import Clock
from twisted.internet.error import ConnectionLost
from twisted.web.xmlrpc import Proxy

from vumi.message import TransportUserMessage
from vumi.tests.helpers import VumiTestCase
from vumi.tests.utils import LogCatcher
from vumi.transports.tests.helpers import TransportHelper
from vumi.transports.trueafrican.transport import TrueAfricanUssdTransport


class TestTrueAfricanUssdTransport(VumiTestCase):

    SESSION_INIT_BODY = {
        'session': '1',
        'msisdn': '+27724385170',
        'shortcode': '*23#'
    }

    @inlineCallbacks
    def setUp(self):
        self.tx_helper = self.add_helper(
            TransportHelper(TrueAfricanUssdTransport))
        self.clock = Clock()
        self.patch(TrueAfricanUssdTransport, 'get_clock', lambda _: self.clock)
        self.transport = yield self.tx_helper.get_transport({
            'interface': '127.0.0.1',
            'port': 0,
            'request_timeout': 10,
        })
        self.service_url = self.get_service_url(self.transport)

    def get_service_url(self, transport):
        """
        Get the URL for the HTTP resource. Requires the worker to be started.
        """
        addr = transport.web_resource.getHost()
        return "http://%s:%s/" % (addr.host, addr.port)

    def web_client(self):
        return Proxy(self.service_url)

    @inlineCallbacks
    def test_session_new(self):
        client = self.web_client()
        resp_d = client.callRemote('USSD.INIT', self.SESSION_INIT_BODY)

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.tx_helper.make_dispatch_reply(msg, "Oh Hai!")

        # verify the transport -> application message
        self.assertEqual(msg['transport_name'], self.tx_helper.transport_name)
        self.assertEqual(msg['transport_type'], "ussd")
        self.assertEqual(msg['session_event'],
                         TransportUserMessage.SESSION_NEW)
        self.assertEqual(msg['from_addr'], '+27724385170')
        self.assertEqual(msg['to_addr'], '*23#')
        self.assertEqual(msg['content'], None)

        resp = yield resp_d
        self.assertEqual(
            resp,
            {
                'message': 'Oh Hai!',
                'session': '1',
                'type': 'cont'
            }
        )

    @inlineCallbacks
    def test_session_resume(self):
        client = self.web_client()

        # initiate session
        resp_d = client.callRemote('USSD.INIT', self.SESSION_INIT_BODY)

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.tx_helper.make_dispatch_reply(msg, "pong")

        yield resp_d
        yield self.tx_helper.clear_dispatched_inbound()

        # resume session
        resp_d = client.callRemote(
            'USSD.CONT',
            {'session': '1',
             'response': 'pong'}
        )

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.tx_helper.make_dispatch_reply(msg, "ping")

        # verify the dispatched inbound message
        self.assertEqual(msg['transport_name'], self.tx_helper.transport_name)
        self.assertEqual(msg['transport_type'], "ussd")
        self.assertEqual(msg['session_event'],
                         TransportUserMessage.SESSION_RESUME)
        self.assertEqual(msg['from_addr'], '+27724385170')
        self.assertEqual(msg['to_addr'], '*23#')
        self.assertEqual(msg['content'], 'pong')

        resp = yield resp_d
        self.assertEqual(
            resp,
            {
                'message': 'ping',
                'session': '1',
                'type': 'cont'
            }
        )

    @inlineCallbacks
    def test_session_end_user_initiated(self):
        client = self.web_client()

        # initiate session
        resp_d = client.callRemote('USSD.INIT', self.SESSION_INIT_BODY)

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.tx_helper.make_dispatch_reply(msg, "ping")

        yield resp_d
        yield self.tx_helper.clear_dispatched_inbound()

        # user initiated session termination
        resp_d = client.callRemote(
            'USSD.END',
            {'session': '1'}
        )

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['transport_name'], self.tx_helper.transport_name)
        self.assertEqual(msg['transport_type'], "ussd")
        self.assertEqual(msg['session_event'],
                         TransportUserMessage.SESSION_CLOSE)
        self.assertEqual(msg['from_addr'], '+27724385170')
        self.assertEqual(msg['to_addr'], '*23#')
        self.assertEqual(msg['content'], None)

        resp = yield resp_d
        self.assertEqual(resp, {})

    @inlineCallbacks
    def test_session_end_application_initiated(self):
        client = self.web_client()

        # initiate session
        resp_d = client.callRemote('USSD.INIT', self.SESSION_INIT_BODY)
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)

        self.tx_helper.make_dispatch_reply(msg, "ping")
        yield resp_d
        yield self.tx_helper.clear_dispatched_inbound()

        # end session
        resp_d = client.callRemote(
            'USSD.CONT',
            {'session': '1',
             'response': 'o rly?'}
        )

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.tx_helper.make_dispatch_reply(
            msg, "kthxbye", continue_session=False)

        resp = yield resp_d
        self.assertEqual(
            resp,
            {
                'message': 'kthxbye',
                'session': '1',
                'type': 'end'
            }
        )

    @inlineCallbacks
    def test_ack_for_outbound_message(self):
        client = self.web_client()

        # initiate session
        resp_d = client.callRemote('USSD.INIT', self.SESSION_INIT_BODY)

        # send response
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        rep = yield self.tx_helper.make_dispatch_reply(msg, "ping")
        yield resp_d

        [ack] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(ack['event_type'], 'ack')
        self.assertEqual(ack['user_message_id'], rep['message_id'])
        self.assertEqual(ack['sent_message_id'], rep['message_id'])

    @inlineCallbacks
    def test_nack_for_outbound_message(self):
        client = self.web_client()

        # initiate session
        resp_d = client.callRemote('USSD.INIT', self.SESSION_INIT_BODY)

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)

        # cancel the request and mute the resulting error.
        request = self.transport._requests[msg['message_id']]
        request.http_request.connectionLost(ConnectionLost())
        resp_d.cancel()
        resp_d.addErrback(lambda f: None)

        # send response
        rep = yield self.tx_helper.make_dispatch_reply(msg, "ping")
        [nack] = yield self.tx_helper.wait_for_dispatched_events(1)

        self.assertEqual(nack['event_type'], 'nack')
        self.assertEqual(nack['user_message_id'], rep['message_id'])
        self.assertEqual(nack['sent_message_id'], rep['message_id'])

    @inlineCallbacks
    def test_nack_for_request_timeout(self):
        client = self.web_client()

        # initiate session
        resp_d = client.callRemote('USSD.INIT', self.SESSION_INIT_BODY)

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)

        self.clock.advance(10.1)  # .1 second after timeout

        rep = yield self.tx_helper.make_dispatch_reply(msg, "ping")
        yield resp_d

        [nack] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(nack['event_type'], 'nack')
        self.assertEqual(nack['user_message_id'], rep['message_id'])
        self.assertEqual(nack['sent_message_id'], rep['message_id'])
        self.assertEqual(nack['nack_reason'], 'Exceeded request timeout')

    @inlineCallbacks
    def test_nack_for_invalid_outbound_message(self):
        msg = yield self.tx_helper.make_dispatch_outbound("outbound")
        [nack] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(nack['user_message_id'], msg['message_id'])
        self.assertEqual(nack['sent_message_id'], msg['message_id'])
        self.assertEqual(nack['nack_reason'],
                         'Missing in_reply_to, content or session_id fields')

    @inlineCallbacks
    def test_timeout(self):
        client = self.web_client()

        # initiate session
        resp_d = client.callRemote('USSD.INIT', self.SESSION_INIT_BODY)

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        with LogCatcher(message='Timing out') as lc:
            self.assertTrue(msg['message_id'] in self.transport._requests)
            self.clock.advance(10.1)  # .1 second after timeout
            self.assertFalse(msg['message_id'] in self.transport._requests)
            [warning] = lc.messages()
            self.assertEqual(warning,
                             'Timing out on response for +27724385170')
            resp = yield resp_d
            self.assertEqual(
                resp,
                {
                    'message': ('We encountered an error while processing'
                                ' your message'),
                    'type': 'end'
                }
            )

    @inlineCallbacks
    def test_request_tracking(self):
        """
        Verify that the transport cleans up after finishing a request
        """
        client = self.web_client()
        resp_d = client.callRemote('USSD.INIT', self.SESSION_INIT_BODY)

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.tx_helper.make_dispatch_reply(msg, "pong")
        self.assertTrue(msg['message_id'] in self.transport._requests)
        yield resp_d
        self.assertFalse(msg['message_id'] in self.transport._requests)

    @inlineCallbacks
    def test_missing_session(self):
        """
        Verify that the transport handles missing session data in a
        graceful manner
        """
        client = self.web_client()
        resp_d = client.callRemote('USSD.INIT', self.SESSION_INIT_BODY)

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.tx_helper.make_dispatch_reply(msg, "pong")
        yield resp_d
        yield self.tx_helper.clear_dispatched_inbound()

        # simulate Redis falling over
        yield self.transport.session_manager.redis._purge_all()

        # resume
        resp_d = client.callRemote(
            'USSD.CONT',
            {'session': '1',
             'response': 'o rly?'}
        )

        resp = yield resp_d
        self.assertEqual(
            resp,
            {
                'message': ('We encountered an error while processing'
                            ' your message'),
                'type': 'end'
            }
        )




from twisted.python import log
from twisted.internet.defer import inlineCallbacks

from vumi.service import Worker


class MessageHandler(object):
    def __init__(self, worker, queue_name, publisher, publish_keys):
        self.worker = worker
        self.queue_name = queue_name
        self.publisher = publisher
        self.publish_keys = publish_keys or []  # if None assume empty set
        log.msg("SimpleDispatcher forwarding from %s to %s" % (
            self.queue_name, self.publish_keys))

    def consume_message(self, message):
        log.msg("SimpleDispatcher consuming on %s: %s" % (
            self.queue_name,
            repr(message)))
        for k in self.publish_keys:
            self.publisher.publish_message(message, routing_key=k)


class SimpleDispatcher(Worker):

    @inlineCallbacks
    def startWorker(self):
        log.msg("Starting SimpleDispatcher with config: %s" % (self.config))

        self.publisher = yield self.publish_to("simpledispatcher.fallback")
        for queue_name, publish_keys in self.config['route_mappings'].items():
            h = MessageHandler(self, queue_name, self.publisher, publish_keys)
            yield self.consume(queue_name, h.consume_message)

    def stopWorker(self):
        log.msg("Stopping SimpleDispatcher")



# -*- test-case-name: vumi.blinkenlights.heartbeat.tests.test_monitor -*-

import time
import collections
import json

from twisted.internet.defer import inlineCallbacks
from twisted.internet.task import LoopingCall

from vumi.worker import BaseWorker
from vumi.config import ConfigDict, ConfigInt
from vumi.blinkenlights.heartbeat.publisher import HeartBeatMessage
from vumi.blinkenlights.heartbeat.storage import Storage
from vumi.persist.txredis_manager import TxRedisManager
from vumi.utils import generate_worker_id
from vumi.errors import ConfigError
from vumi import log


WorkerIssue = collections.namedtuple('WorkerIssue',
                                     ['issue_type',
                                      'start_time',
                                      'procs_count'])


def assert_field(cfg, key):
    """
    helper to check whether a config key is defined. Only used for
    verifying dict fields in the new-style configs
    """
    if key not in cfg:
        raise ConfigError("Expected '%s' field in config" % key)


class WorkerInstance(object):
    """Represents a worker instance.

    A hostname, process id pair uniquely identify a worker instance.
    """

    def __init__(self, hostname, pid):
        self.hostname = hostname
        self.pid = pid

    def __eq__(self, obj):
        if not isinstance(obj, WorkerInstance):
            return NotImplemented
        return (self.hostname == obj.hostname and
                self.pid == obj.pid)

    def __hash__(self):
        return hash((self.hostname, self.pid))


class Worker(object):

    def __init__(self, system_id, worker_name, min_procs):
        self.system_id = system_id
        self.name = worker_name
        self.min_procs = min_procs
        self.worker_id = generate_worker_id(system_id, worker_name)
        self._instances = set()
        self._instances_active = set()
        self.procs_count = 0

    def to_dict(self):
        """Serializes information into basic dicts"""
        counts = self._compute_host_info(self._instances)
        hosts = []
        for host, count in counts.iteritems():
            hosts.append({
                    'host': host,
                    'proc_count': count,
            })
        obj = {
            'id': self.worker_id,
            'name': self.name,
            'system_id': self.system_id,
            'min_procs': self.min_procs,
            'hosts': hosts,
        }
        return obj

    def _compute_host_info(self, instances):
        """Compute the number of worker instances running on each host."""
        counts = {}
        # initialize per-host counters
        for ins in instances:
            counts[ins.hostname] = 0
        # update counters for each instance
        for ins in instances:
            counts[ins.hostname] = counts[ins.hostname] + 1
        return counts

    @inlineCallbacks
    def audit(self, storage):
        """
        Verify whether enough workers checked in.
        Make sure to call snapshot() before running this method
        """
        count = len(self._instances)
        # if there was previously a min-procs-fail, but now enough
        # instances checked in, then clear the worker issue
        if (count >= self.min_procs) and (self.procs_count < self.min_procs):
            yield storage.delete_worker_issue(self.worker_id)
        if count < self.min_procs:
            issue = WorkerIssue("min-procs-fail", time.time(), count)
            yield storage.open_or_update_issue(self.worker_id, issue)
        self.procs_count = count

    def snapshot(self):
        """
        This method must be run before any diagnostic audit and analyses

        What it does is clear the instances_active set in preparation for
        all the instances which will check-in in the next interval.

        All diagnostics are based on the _instances_active set, which
        holds all the instances which checked-in the previous interval.
        """
        self._instances = self._instances_active
        self._instances_active = set()

    def record(self, hostname, pid):
        """Record that process (hostname,pid) checked in."""
        self._instances_active.add(WorkerInstance(hostname, pid))


class System(object):

    def __init__(self, system_name, system_id, workers):
        self.name = system_name
        self.system_id = system_id
        self.workers = workers

    def to_dict(self):
        """Serialize information to basic dicts"""
        obj = {
            'name': self.name,
            'id': self.system_id,
            'timestamp': int(time.time()),
            'workers': [wkr.to_dict() for wkr in self.workers],
        }
        return obj

    def dumps(self):
        """Dump to a JSON string"""
        return json.dumps(self.to_dict())


class HeartBeatMonitor(BaseWorker):

    class CONFIG_CLASS(BaseWorker.CONFIG_CLASS):
        deadline = ConfigInt(
            "Check-in deadline for participating workers",
            required=True, static=True)
        redis_manager = ConfigDict(
            "Redis client configuration.",
            required=True, static=True)
        monitored_systems = ConfigDict(
            "Tree of systems and workers.",
            required=True, static=True)

    _task = None

    @inlineCallbacks
    def startWorker(self):
        log.msg("Heartbeat monitor initializing")
        config = self.get_static_config()

        self.deadline = config.deadline

        redis_config = config.redis_manager
        self._redis = yield TxRedisManager.from_config(redis_config)
        self._storage = Storage(self._redis)

        self._systems, self._workers = self.parse_config(
            config.monitored_systems)

        # Start consuming heartbeats
        yield self.consume("heartbeat.inbound", self._consume_message,
                           exchange_name='vumi.health',
                           message_class=HeartBeatMessage)

        self._start_task()

    @inlineCallbacks
    def stopWorker(self):
        log.msg("HeartBeat: stopping worker")
        if self._task:
            self._task.stop()
            self._task = None
            yield self._task_done
        self._redis.close_manager()

    def parse_config(self, config):
        """
        Parse configuration and populate in-memory state
        """
        systems = []
        workers = {}
        # loop over each defined system
        for sys in config.values():
            assert_field(sys, 'workers')
            assert_field(sys, 'system_id')
            system_id = sys['system_id']
            system_workers = []
            # loop over each defined worker in the system
            for wkr_entry in sys['workers'].values():
                assert_field(wkr_entry, 'name')
                assert_field(wkr_entry, 'min_procs')
                worker_name = wkr_entry['name']
                min_procs = wkr_entry['min_procs']
                wkr = Worker(system_id,
                             worker_name,
                             min_procs)
                workers[wkr.worker_id] = wkr
                system_workers.append(wkr)
            systems.append(System(system_id, system_id, system_workers))
        return systems, workers

    def update(self, msg):
        """
        Process a heartbeat message.
        """
        worker_id = msg['worker_id']
        timestamp = msg['timestamp']
        hostname = msg['hostname']
        pid = msg['pid']

        # A bunch of discard rules:
        # 1. Unknown worker (Monitored workers need to be in the config)
        # 2. Message which are too old.
        wkr = self._workers.get(worker_id, None)
        if wkr is None:
            log.msg("Discarding message. worker '%s' is unknown" % worker_id)
            return
        if timestamp < (time.time() - self.deadline):
            log.msg("Discarding heartbeat from '%s'. Too old" % worker_id)
            return

        wkr.record(hostname, pid)

    @inlineCallbacks
    def _sync_to_storage(self):
        """
        Write systems data to storage
        """
        # write system ids
        system_ids = [sys.system_id for sys in self._systems]
        yield self._storage.add_system_ids(system_ids)
        # dump each system
        for sys in self._systems:
            yield self._storage.write_system(sys)

    @inlineCallbacks
    def _periodic_task(self):
        """
        Iterate over worker instance sets and check to see whether any have not
        checked-in on time.

        We call snapshot() first, since the execution of tasks here is
        interleaved with the processing of worker heartbeat messages.
        """
        # snapshot the the set of checked-in instances
        for wkr in self._workers.values():
            wkr.snapshot()
        # run diagnostic audits on all workers
        for wkr in self._workers.values():
            yield wkr.audit(self._storage)
        # write everything to redis
        yield self._sync_to_storage()

    def _start_task(self):
        """Create a timer task to check for missing worker"""
        self._task = LoopingCall(self._periodic_task)
        self._task_done = self._task.start(self.deadline, now=False)
        errfn = lambda failure: log.err(failure,
                                        "Heartbeat verify: timer task died")
        self._task_done.addErrback(errfn)

    def _consume_message(self, msg):
        log.msg("Received message: %s" % msg)
        self.update(msg.payload)



from vumi.transports.xmpp import xmpp


class TestXMLStream(object):

    def __init__(self):
        self.outbox = []

    def send(self, message):
        self.outbox.append(message)

    def addObserver(self, event, observerfn, *args, **kwargs):
        """Ignore."""


class TestXMPPClient(xmpp.XMPPClient):
    def __init__(self, *args, **kw):
        xmpp.XMPPClient.__init__(self, *args, **kw)
        self._connection = None

    def startService(self):
        pass

    def stopService(self):
        pass


class TestXMPPTransportProtocol(xmpp.XMPPTransportProtocol):

    def __init__(self, *args, **kwargs):
        xmpp.XMPPTransportProtocol.__init__(self, *args, **kwargs)
        self.xmlstream = TestXMLStream()



import json
from urllib import urlencode

from twisted.internet.defer import inlineCallbacks

from vumi.message import TransportUserMessage
from vumi.tests.helpers import VumiTestCase
from vumi.transports.safaricom import SafaricomTransport
from vumi.transports.tests.helpers import TransportHelper
from vumi.utils import http_request


class TestSafaricomTransport(VumiTestCase):

    @inlineCallbacks
    def setUp(self):
        config = {
            'web_port': 0,
            'web_path': '/api/v1/safaricom/ussd/',
        }
        self.tx_helper = self.add_helper(TransportHelper(SafaricomTransport))
        self.transport = yield self.tx_helper.get_transport(config)
        self.session_manager = self.transport.session_manager
        self.transport_url = self.transport.get_transport_url(
            config['web_path'])
        yield self.session_manager.redis._purge_all()  # just in case

    def mk_full_request(self, **params):
        return http_request('%s?%s' % (self.transport_url,
            urlencode(params)), data='', method='GET')

    def mk_request(self, **params):
        defaults = {
            'ORIG': '27761234567',
            'DEST': '167',
            'SESSION_ID': 'session-id',
            'USSD_PARAMS': '',
        }
        defaults.update(params)
        return self.mk_full_request(**defaults)

    @inlineCallbacks
    def test_inbound_begin(self):
        # Second connect is the actual start of the session
        deferred = self.mk_request(USSD_PARAMS='7')
        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], '')
        self.assertEqual(msg['to_addr'], '*167*7#')
        self.assertEqual(msg['from_addr'], '27761234567'),
        self.assertEqual(msg['session_event'],
                         TransportUserMessage.SESSION_NEW)
        self.assertEqual(msg['transport_metadata'], {
            'safaricom': {
                'session_id': 'session-id',
            },
        })

        yield self.tx_helper.make_dispatch_reply(msg, "ussd message")
        response = yield deferred
        self.assertEqual(response, 'CON ussd message')

    @inlineCallbacks
    def test_inbound_resume_and_reply_with_end(self):
        # first pre-populate the redis datastore to simulate prior BEG message
        yield self.session_manager.create_session('session-id',
                to_addr='*167*7#', from_addr='27761234567',
                last_ussd_params='7*a*b',
                session_event=TransportUserMessage.SESSION_RESUME)

        # Safaricom gives us the history of the full session in the USSD_PARAMS
        # The last submitted bit of content is the last value delimited by '*'
        deferred = self.mk_request(USSD_PARAMS='7*a*b*c')

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], 'c')
        self.assertEqual(msg['to_addr'], '*167*7#')
        self.assertEqual(msg['from_addr'], '27761234567')
        self.assertEqual(msg['session_event'],
                         TransportUserMessage.SESSION_RESUME)
        self.assertEqual(msg['transport_metadata'], {
            'safaricom': {
                'session_id': 'session-id',
            },
        })

        yield self.tx_helper.make_dispatch_reply(
            msg, "hello world", continue_session=False)
        response = yield deferred
        self.assertEqual(response, 'END hello world')

    @inlineCallbacks
    def test_inbound_resume_with_failed_to_addr_lookup(self):
        deferred = self.mk_full_request(ORIG='123456',
            USSD_PARAMS='7*a', SESSION_ID='session-id')
        response = yield deferred
        self.assertEqual(json.loads(response), {
            'missing_parameter': ['DEST'],
        })

    @inlineCallbacks
    def test_to_addr_handling(self):
        defaults = {
            'DEST': '167',
            'ORIG': '12345',
            'SESSION_ID': 'session-id',
        }

        d1 = self.mk_full_request(USSD_PARAMS='7*1', **defaults)
        [msg1] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg1['to_addr'], '*167*7*1#')
        self.assertEqual(msg1['content'], '')
        self.assertEqual(msg1['session_event'],
            TransportUserMessage.SESSION_NEW)
        yield self.tx_helper.make_dispatch_reply(msg1, "hello world")
        yield d1

        # follow up with the user submitting 'a'
        d2 = self.mk_full_request(USSD_PARAMS='7*1*a', **defaults)
        [msg1, msg2] = yield self.tx_helper.wait_for_dispatched_inbound(2)
        self.assertEqual(msg2['to_addr'], '*167*7*1#')
        self.assertEqual(msg2['content'], 'a')
        self.assertEqual(msg2['session_event'],
            TransportUserMessage.SESSION_RESUME)
        yield self.tx_helper.make_dispatch_reply(
            msg2, "hello world", continue_session=False)
        yield d2

    @inlineCallbacks
    def test_hitting_url_twice_without_content(self):
        d1 = self.mk_request(USSD_PARAMS='7*3')
        [msg1] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg1['to_addr'], '*167*7*3#')
        self.assertEqual(msg1['content'], '')
        self.assertEqual(msg1['session_event'],
            TransportUserMessage.SESSION_NEW)
        yield self.tx_helper.make_dispatch_reply(msg1, "Hello")
        yield d1

        # make the exact same request again
        d2 = self.mk_request(USSD_PARAMS='7*3')
        [msg1, msg2] = yield self.tx_helper.wait_for_dispatched_inbound(2)
        self.assertEqual(msg2['to_addr'], '*167*7*3#')
        self.assertEqual(msg2['content'], '')
        self.assertEqual(msg2['session_event'],
            TransportUserMessage.SESSION_RESUME)
        yield self.tx_helper.make_dispatch_reply(msg2, "Hello")
        yield d2

    @inlineCallbacks
    def test_submitting_asterisks_as_values(self):
        yield self.session_manager.create_session('session-id',
                to_addr='*167*7#', from_addr='27761234567',
                last_ussd_params='7*a*b')
        # we're submitting a bunch of *s
        deferred = self.mk_request(USSD_PARAMS='7*a*b*****')

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], '****')

        yield self.tx_helper.make_dispatch_reply(msg, "Hello")
        yield deferred
        session = yield self.session_manager.load_session('session-id')
        self.assertEqual(session['last_ussd_params'], '7*a*b*****')

    @inlineCallbacks
    def test_submitting_asterisks_as_values_after_asterisks(self):
        yield self.session_manager.create_session('session-id',
                to_addr='*167*7#', from_addr='27761234567',
                last_ussd_params='7*a*b**')
        # we're submitting a bunch of *s
        deferred = self.mk_request(USSD_PARAMS='7*a*b*****')

        [msg] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg['content'], '**')

        yield self.tx_helper.make_dispatch_reply(msg, "Hello")
        yield deferred
        session = yield self.session_manager.load_session('session-id')
        self.assertEqual(session['last_ussd_params'], '7*a*b*****')

    @inlineCallbacks
    def test_submitting_with_base_code_empty_ussd_params(self):
        d1 = self.mk_request()
        [msg1] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(msg1['to_addr'], '*167#')
        self.assertEqual(msg1['content'], '')
        self.assertEqual(msg1['session_event'],
            TransportUserMessage.SESSION_NEW)
        yield self.tx_helper.make_dispatch_reply(msg1, "Hello")
        yield d1

        # ask for first menu
        d2 = self.mk_request(USSD_PARAMS='1')
        [msg1, msg2] = yield self.tx_helper.wait_for_dispatched_inbound(2)
        self.assertEqual(msg2['to_addr'], '*167#')
        self.assertEqual(msg2['content'], '1')
        self.assertEqual(msg2['session_event'],
            TransportUserMessage.SESSION_RESUME)
        yield self.tx_helper.make_dispatch_reply(msg2, "Hello")
        yield d2

        # ask for second menu
        d3 = self.mk_request(USSD_PARAMS='1*1')
        [m1, m2, msg3] = yield self.tx_helper.wait_for_dispatched_inbound(3)
        self.assertEqual(msg3['to_addr'], '*167#')
        self.assertEqual(msg3['content'], '1')
        self.assertEqual(msg3['session_event'],
            TransportUserMessage.SESSION_RESUME)
        yield self.tx_helper.make_dispatch_reply(msg3, "Hello")
        yield d3

    @inlineCallbacks
    def test_nack(self):
        msg = yield self.tx_helper.make_dispatch_outbound("outbound")
        [nack] = yield self.tx_helper.wait_for_dispatched_events(1)
        self.assertEqual(nack['user_message_id'], msg['message_id'])
        self.assertEqual(nack['sent_message_id'], msg['message_id'])
        self.assertEqual(nack['nack_reason'],
            'Missing fields: in_reply_to')



from twisted.internet.defer import inlineCallbacks

from smpp.pdu_builder import DeliverSM

from vumi.message import TransportUserMessage

from vumi.transports.smpp.pdu_utils import (
    command_id, seq_no, pdu_tlv, short_message)
from vumi.transports.smpp.tests.test_smpp_transport import (
    SmppTransportTestCase)
from vumi.transports.smpp.smpp_transport import (
    SmppTransceiverTransport)
from vumi.transports.smpp.processors.mica import make_vumi_session_identifier


class MicaProcessorTestCase(SmppTransportTestCase):

    transport_class = SmppTransceiverTransport

    def setUp(self):
        super(MicaProcessorTestCase, self).setUp()
        self.default_config = {
            'transport_name': self.tx_helper.transport_name,
            'twisted_endpoint': 'tcp:host=127.0.0.1:port=0',
            'deliver_short_message_processor': (
                'vumi.transports.smpp.processors.mica.'
                'DeliverShortMessageProcessor'),
            'submit_short_message_processor': (
                'vumi.transports.smpp.processors.mica.'
                'SubmitShortMessageProcessor'),
            'system_id': 'foo',
            'password': 'bar',
            'deliver_short_message_processor_config': {
                'data_coding_overrides': {
                    0: 'utf-8',
                }
            },
            'submit_short_message_processor_config': {
                'submit_sm_encoding': 'utf-16be',
                'submit_sm_data_coding': 8,
                'send_multipart_udh': True,
            }
        }

    def assert_udh_parts(self, pdus, texts, encoding):
        pdu_header = lambda pdu: short_message(pdu)[:6]
        pdu_text = lambda pdu: short_message(pdu)[6:].decode(encoding)
        udh_header = lambda i: '\x05\x00\x03\x03\x07' + chr(i)
        self.assertEqual(
            [(pdu_header(pdu), pdu_text(pdu)) for pdu in pdus],
            [(udh_header(i + 1), text) for i, text in enumerate(texts)])

    @inlineCallbacks
    def test_submit_sm_multipart_udh_ucs2(self):
        message = (
            "A cup is a small, open container used for carrying and "
            "drinking drinks. It may be made of wood, plastic, glass, "
            "clay, metal, stone, china or other materials, and may have "
            "a stem, handles or other adornments. Cups are used for "
            "drinking across a wide range of cultures and social classes, "
            "and different styles of cups may be used for different liquids "
            "or in different situations. Cups have been used for thousands "
            "of years for the ...Reply 1 for more")

        smpp_helper = yield self.get_smpp_helper()
        yield self.tx_helper.make_dispatch_outbound(message, to_addr='msisdn')
        pdus = yield smpp_helper.wait_for_pdus(7)
        self.assert_udh_parts(pdus, [
            ("A cup is a small, open container used"
             " for carrying and drinking d"),
            ("rinks. It may be made of wood, plastic,"
             " glass, clay, metal, stone"),
            (", china or other materials, and may have"
             " a stem, handles or other"),
            (" adornments. Cups are used for drinking"
             " across a wide range of cu"),
            ("ltures and social classes, and different"
             " styles of cups may be us"),
            ("ed for different liquids or in different"
             " situations. Cups have be"),
            ("en used for thousands of years for the ...Reply 1 for more"),
        ], encoding='utf-16be')  # utf-16be is close enough to UCS2
        for pdu in pdus:
            self.assertTrue(len(short_message(pdu)) < 140)

    @inlineCallbacks
    def test_submit_and_deliver_ussd_new(self):
        session_identifier = 12345
        smpp_helper = yield self.get_smpp_helper()

        # Server delivers a USSD message to the Client
        pdu = DeliverSM(1, short_message="*123#")
        pdu.add_optional_parameter('ussd_service_op', '01')
        pdu.add_optional_parameter('user_message_reference',
                                   session_identifier)

        yield smpp_helper.handle_pdu(pdu)

        [mess] = yield self.tx_helper.wait_for_dispatched_inbound(1)

        self.assertEqual(mess['content'], None)
        self.assertEqual(mess['to_addr'], '*123#')
        self.assertEqual(mess['transport_type'], "ussd")
        self.assertEqual(mess['session_event'],
                         TransportUserMessage.SESSION_NEW)
        self.assertEqual(
            mess['transport_metadata'],
            {
                'session_info': {
                    'session_identifier': 12345,
                    'ussd_service_op': '01',
                }
            })

    @inlineCallbacks
    def test_deliver_sm_op_codes_new(self):
        session_identifier = 12345
        smpp_helper = yield self.get_smpp_helper()
        pdu = DeliverSM(1, short_message="*123#")
        pdu.add_optional_parameter('ussd_service_op', '01')
        pdu.add_optional_parameter('user_message_reference',
                                   session_identifier)
        yield smpp_helper.handle_pdu(pdu)
        [start] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(start['session_event'],
                         TransportUserMessage.SESSION_NEW)

    @inlineCallbacks
    def test_deliver_sm_op_codes_resume(self):
        source_addr = 'msisdn'
        session_identifier = 12345
        vumi_session_identifier = make_vumi_session_identifier(
            source_addr, session_identifier)

        smpp_helper = yield self.get_smpp_helper()
        deliver_sm_processor = smpp_helper.transport.deliver_sm_processor
        session_manager = deliver_sm_processor.session_manager

        yield session_manager.create_session(
            vumi_session_identifier, ussd_code='*123#')

        pdu = DeliverSM(1, short_message="", source_addr=source_addr)
        pdu.add_optional_parameter('ussd_service_op', '12')
        pdu.add_optional_parameter('user_message_reference',
                                   session_identifier)
        yield smpp_helper.handle_pdu(pdu)
        [resume] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(resume['session_event'],
                         TransportUserMessage.SESSION_RESUME)

    @inlineCallbacks
    def test_deliver_sm_op_codes_end(self):
        source_addr = 'msisdn'
        session_identifier = 12345
        vumi_session_identifier = make_vumi_session_identifier(
            source_addr, session_identifier)

        smpp_helper = yield self.get_smpp_helper()
        deliver_sm_processor = smpp_helper.transport.deliver_sm_processor
        session_manager = deliver_sm_processor.session_manager

        yield session_manager.create_session(
            vumi_session_identifier, ussd_code='*123#')

        pdu = DeliverSM(1, short_message="", source_addr=source_addr)
        pdu.add_optional_parameter('ussd_service_op', '81')
        pdu.add_optional_parameter('user_message_reference',
                                   session_identifier)
        yield smpp_helper.handle_pdu(pdu)
        [end] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(end['session_event'],
                         TransportUserMessage.SESSION_CLOSE)

    @inlineCallbacks
    def test_deliver_sm_unknown_op_code(self):
        session_identifier = 12345
        smpp_helper = yield self.get_smpp_helper()

        pdu = DeliverSM(1, short_message="*123#")
        pdu.add_optional_parameter('ussd_service_op', '01')
        pdu.add_optional_parameter('user_message_reference',
                                   session_identifier)

        yield smpp_helper.handle_pdu(pdu)

        pdu = DeliverSM(1, short_message="*123#")
        pdu.add_optional_parameter('ussd_service_op', '99')
        pdu.add_optional_parameter('user_message_reference',
                                   session_identifier)

        yield smpp_helper.handle_pdu(pdu)
        [start, unknown] = yield self.tx_helper.wait_for_dispatched_inbound(1)
        self.assertEqual(unknown['session_event'],
                         TransportUserMessage.SESSION_RESUME)

    @inlineCallbacks
    def test_submit_sm_op_codes_resume(self):
        user_msisdn = 'msisdn'
        session_identifier = 12345
        smpp_helper = yield self.get_smpp_helper()

        yield self.tx_helper.make_dispatch_outbound(
            "hello world",
            transport_type="ussd",
            session_event=TransportUserMessage.SESSION_RESUME,
            transport_metadata={
                'session_info': {
                    'session_identifier': session_identifier
                }
            }, to_addr=user_msisdn)
        [resume] = yield smpp_helper.wait_for_pdus(1)
        self.assertEqual(pdu_tlv(resume, 'ussd_service_op'), '02')

    @inlineCallbacks
    def test_submit_sm_op_codes_close(self):
        user_msisdn = 'msisdn'
        session_identifier = 12345
        smpp_helper = yield self.get_smpp_helper()

        yield self.tx_helper.make_dispatch_outbound(
            "hello world",
            transport_type="ussd",
            session_event=TransportUserMessage.SESSION_CLOSE,
            transport_metadata={
                'session_info': {
                    'session_identifier': session_identifier
                }
            }, to_addr=user_msisdn)

        [close] = yield smpp_helper.wait_for_pdus(1)
        self.assertEqual(pdu_tlv(close, 'ussd_service_op'), '17')

    @inlineCallbacks
    def test_submit_and_deliver_ussd_continue(self):
        user_msisdn = 'msisdn'
        session_identifier = 12345
        vumi_session_identifier = make_vumi_session_identifier(
            user_msisdn, session_identifier)
        smpp_helper = yield self.get_smpp_helper()

        deliver_sm_processor = smpp_helper.transport.deliver_sm_processor
        session_manager = deliver_sm_processor.session_manager
        yield session_manager.create_session(
            vumi_session_identifier, ussd_code='*123#')

        yield self.tx_helper.make_dispatch_outbound(
            "hello world", transport_type="ussd", transport_metadata={
                'session_info': {
                    'session_identifier': session_identifier
                }
            }, to_addr=user_msisdn)

        [submit_sm_pdu] = yield smpp_helper.wait_for_pdus(1)
        self.assertEqual(command_id(submit_sm_pdu), 'submit_sm')
        self.assertEqual(pdu_tlv(submit_sm_pdu, 'ussd_service_op'), '02')
        self.assertEqual(
            pdu_tlv(submit_sm_pdu, 'user_message_reference'),
            session_identifier)

        # Server delivers a USSD message to the Client
        pdu = DeliverSM(seq_no(submit_sm_pdu) + 1, short_message="reply!",
                        source_addr=user_msisdn)
        # 0x12 is 'continue'
        pdu.add_optional_parameter('ussd_service_op', '12')
        pdu.add_optional_parameter('user_message_reference',
                                   session_identifier)

        yield smpp_helper.handle_pdu(pdu)

        [mess] = yield self.tx_helper.wait_for_dispatched_inbound(1)

        self.assertEqual(mess['content'], "reply!")
        self.assertEqual(mess['transport_type'], "ussd")
        self.assertEqual(mess['to_addr'], '*123#')
        self.assertEqual(mess['session_event'],
                         TransportUserMessage.SESSION_RESUME)

    @inlineCallbacks
    def test_submit_and_deliver_ussd_close(self):
        smpp_helper = yield self.get_smpp_helper()
        session_identifier = 12345

        yield self.tx_helper.make_dispatch_outbound(
            "hello world", transport_type="ussd",
            session_event=TransportUserMessage.SESSION_CLOSE,
            transport_metadata={
                'session_info': {
                    'session_identifier': session_identifier
                }
            })

        [submit_sm_pdu] = yield smpp_helper.wait_for_pdus(1)
        self.assertEqual(command_id(submit_sm_pdu), 'submit_sm')
        self.assertEqual(pdu_tlv(submit_sm_pdu, 'ussd_service_op'), '17')
        self.assertEqual(pdu_tlv(submit_sm_pdu, 'user_message_reference'),
                         session_identifier)

    @inlineCallbacks
    def test_submit_sm_null_message(self):
        """
        We can successfully send a message with null content.
        """
        user_msisdn = 'msisdn'
        session_identifier = 12345
        smpp_helper = yield self.get_smpp_helper()

        yield self.tx_helper.make_dispatch_outbound(
            None,
            transport_type="ussd",
            session_event=TransportUserMessage.SESSION_RESUME,
            transport_metadata={
                'session_info': {
                    'session_identifier': session_identifier
                }
            }, to_addr=user_msisdn)
        [resume] = yield smpp_helper.wait_for_pdus(1)
        self.assertEqual(pdu_tlv(resume, 'ussd_service_op'), '02')



# -*- test-case-name: vumi.transports.vas2nets.tests.test_vas2nets_stubs -*-

import uuid
import random
from urllib import urlencode
from StringIO import StringIO
from urlparse import urlparse
from datetime import datetime

from twisted.python import log
from twisted.internet.defer import inlineCallbacks, Deferred, succeed
from twisted.internet import reactor
from twisted.internet.protocol import Protocol
from twisted.web import http
from twisted.web.resource import Resource
from twisted.web.client import Agent
from twisted.web.http_headers import Headers

from vumi.utils import StringProducer
from vumi.service import Worker


class HttpResponseHandler(Protocol):
    def __init__(self, deferred):
        self.deferred = deferred
        self.stringio = StringIO()

    def dataReceived(self, bytes):
        self.stringio.write(bytes)

    def connectionLost(self, reason):
        self.deferred.callback(self.stringio.getvalue())

    @classmethod
    def handle(cls, response):
        deferred = Deferred()
        response.deliverBody(cls(deferred))
        return deferred

    @classmethod
    def req_POST(cls, url, params, headers=None):
        agent = Agent(reactor)
        hdrs = {
            'User-Agent': ['Vumi Vas2Net Faker'],
            'Content-Type': ['application/x-www-form-urlencoded'],
            }
        if headers:
            hdrs.update(headers)
        d = agent.request('POST', url, Headers(hdrs),
                          StringProducer(urlencode(params)))
        return d.addCallback(cls.handle)


class FakeVas2NetsHandler(Resource):
    """
    Resource to accept outgoing messages and reply with a delivery report.
    """
    isLeaf = True

    delay_choices = (0.5, 1, 1.5, 2, 5)
    deliver_hook = None

    def __init__(self, receipt_url, delay_choices=None, deliver_hook=None):
        if delay_choices:
            self.delay_choices = delay_choices
        if deliver_hook:
            self.deliver_hook = deliver_hook
        self.receipt_url = receipt_url

    def get_sms_id(self):
        return uuid.uuid4().get_hex()[-8:]

    def render_POST(self, request):
        request.setResponseCode(http.OK)
        required_fields = [
            'username', 'password', 'call-number', 'origin', 'text',
            'messageid', 'provider', 'tariff', 'owner', 'service',
            'subservice'
            ]
        log.msg('Received sms: %s' % (request.args,))
        for key in required_fields:
            if key not in request.args:
                request.setResponseCode(http.BAD_REQUEST)

        sms_id = self.get_sms_id()
        request.setHeader('X-Nth-Smsid', sms_id)
        self.schedule_delivery(sms_id, *[request.args.get(f) for f in
                                         ['messageid', 'provider', 'sender']])
        return "Result_code: 00, Message OK"

    def schedule_delivery(self, *args):
        if not self.receipt_url:
            return succeed(None)
        return reactor.callLater(random.choice(self.delay_choices),
                                 self.deliver_receipt, *args)

    def deliver_receipt(self, sms_id, message_id, provider, sender):
        CODES = [
            ('2', 'DELIVRD'),
            ('2', 'DELIVRD'),
            ('2', 'DELIVRD'),
            ('2', 'DELIVRD'),
            ('-28', 'Presumably failed.'),
            ]
        code, status = random.choice(CODES)
        params = {
            'smsid': sms_id,
            'messageid': message_id,
            'status': code,
            'text': status,
            'time': datetime.utcnow().strftime('%Y.%m.%d %H:%M:%S'),
            'provider': provider,
            'sender': sender,
            }

        log.msg("Sending receipt: %s" % (params,))
        d = HttpResponseHandler.req_POST(self.receipt_url, params)
        if self.deliver_hook:
            d.addCallback(self.deliver_hook)
        return d


class FakeVas2NetsWorker(Worker):
    delay_choices = None
    deliver_hook = None

    handler = FakeVas2NetsHandler

    @inlineCallbacks
    def startWorker(self):
        url = urlparse(self.config.get('url'))
        receipt_url = "http://127.0.0.1:%s%s" % (
            self.config.get('web_port'), self.config.get('web_receipt_path'))

        self.receipt_resource = yield self.start_web_resources(
            [(self.handler(receipt_url, self.delay_choices, self.deliver_hook),
              url.path)], url.port)

    def stopWorker(self):
        if hasattr(self, 'receipt_resource'):
            self.receipt_resource.stopListening()



# -*- test-case-name: vumi.blinkenlights.tests.test_metrics_workers -*-

import time
import random
import hashlib
from datetime import datetime

from twisted.python import log
from twisted.internet.defer import inlineCallbacks, Deferred
from twisted.internet import reactor
from twisted.internet.task import LoopingCall
from twisted.internet.protocol import DatagramProtocol

from vumi.service import Consumer, Publisher, Worker
from vumi.blinkenlights.metrics import (MetricsConsumer, MetricManager, Count,
                                        Metric, Timer, Aggregator)
from vumi.blinkenlights.message20110818 import MetricMessage


class AggregatedMetricConsumer(Consumer):
    """Consumer for aggregate metrics.

    Parameters
    ----------
    callback : function (metric_name, values)
        Called for each metric datapoint as it arrives.  The
        parameters are metric_name (str) and values (a list of
        timestamp and value pairs).
    """
    exchange_name = "vumi.metrics.aggregates"
    exchange_type = "direct"
    durable = True
    routing_key = "vumi.metrics.aggregates"

    def __init__(self, channel, callback):
        self.queue_name = self.routing_key
        super(AggregatedMetricConsumer, self).__init__(channel)
        self.callback = callback

    def consume_message(self, vumi_message):
        msg = MetricMessage.from_dict(vumi_message.payload)
        for metric_name, _aggregators, values in msg.datapoints():
            self.callback(metric_name, values)


class AggregatedMetricPublisher(Publisher):
    """Publishes aggregated metrics.
    """
    exchange_name = "vumi.metrics.aggregates"
    exchange_type = "direct"
    durable = True
    routing_key = "vumi.metrics.aggregates"

    def publish_aggregate(self, metric_name, timestamp, value):
        # TODO: perhaps change interface to publish multiple metrics?
        msg = MetricMessage()
        msg.append((metric_name, (), [(timestamp, value)]))
        self.publish_message(msg)


class TimeBucketConsumer(Consumer):
    """Consume time bucketed metric messages.

    Parameters
    ----------
    bucket : int
        Bucket to consume time buckets from.
    callback : function, f(metric_name, aggregators, values)
        Called for each metric datapoint as it arrives.
        The parameters are metric_name (str),
        aggregator (list of aggregator names) and values (a
        list of timestamp and value pairs).
    """
    exchange_name = "vumi.metrics.buckets"
    exchange_type = "direct"
    durable = True
    ROUTING_KEY_TEMPLATE = "bucket.%d"

    def __init__(self, channel, bucket, callback):
        self.queue_name = self.ROUTING_KEY_TEMPLATE % bucket
        self.routing_key = self.queue_name
        super(TimeBucketConsumer, self).__init__(channel)
        self.callback = callback

    def consume_message(self, vumi_message):
        msg = MetricMessage.from_dict(vumi_message.payload)
        for metric_name, aggregators, values in msg.datapoints():
            self.callback(metric_name, aggregators, values)


class TimeBucketPublisher(Publisher):
    """Publish time bucketed metric messages.

    Parameters
    ----------
    buckets : int
        Total number of buckets messages are being
        distributed to.
    bucket_size : int, in seconds
        Size of each time bucket in seconds.
    """
    exchange_name = "vumi.metrics.buckets"
    exchange_type = "direct"
    durable = True
    ROUTING_KEY_TEMPLATE = "bucket.%d"

    def __init__(self, buckets, bucket_size):
        self.buckets = buckets
        self.bucket_size = bucket_size

    def find_bucket(self, metric_name, ts_key):
        md5 = hashlib.md5("%s:%d" % (metric_name, ts_key))
        return int(md5.hexdigest(), 16) % self.buckets

    def publish_metric(self, metric_name, aggregates, values):
        timestamp_buckets = {}
        for timestamp, value in values:
            ts_key = int(timestamp) / self.bucket_size
            ts_bucket = timestamp_buckets.get(ts_key)
            if ts_bucket is None:
                ts_bucket = timestamp_buckets[ts_key] = []
            ts_bucket.append((timestamp, value))

        for ts_key, ts_bucket in timestamp_buckets.iteritems():
            bucket = self.find_bucket(metric_name, ts_key)
            routing_key = self.ROUTING_KEY_TEMPLATE % bucket
            msg = MetricMessage()
            msg.append((metric_name, aggregates, ts_bucket))
            self.publish_message(msg, routing_key=routing_key)


class MetricTimeBucket(Worker):
    """Gathers metrics messages and redistributes them to aggregators.

    :class:`MetricTimeBuckets` take metrics from the vumi.metrics
    exchange and redistribute them to one of N :class:`MetricAggregator`
    workers.

    There can be any number of :class:`MetricTimeBucket` workers.

    Configuration Values
    --------------------
    buckets : int (N)
        The total number of aggregator workers. :class:`MetricAggregator`
        workers must be started with bucket numbers 0 to N-1 otherwise
        metric data will go missing (or at best be stuck in a queue
        somewhere).
    bucket_size : int, in seconds
        The amount of time each time bucket represents.
    """
    @inlineCallbacks
    def startWorker(self):
        log.msg("Starting a MetricTimeBucket with config: %s" % self.config)
        buckets = int(self.config.get("buckets"))
        log.msg("Total number of buckets %d" % buckets)
        bucket_size = int(self.config.get("bucket_size"))
        log.msg("Bucket size is %d seconds" % bucket_size)
        self.publisher = yield self.start_publisher(TimeBucketPublisher,
                                                    buckets, bucket_size)
        self.consumer = yield self.start_consumer(MetricsConsumer,
                self.publisher.publish_metric)


class DiscardedMetricError(Exception):
    pass


class MetricAggregator(Worker):
    """Gathers a subset of metrics and aggregates them.

    :class:`MetricAggregators` work in sets of N.

    Configuration Values
    --------------------
    bucket : int, 0 to N-1
        An aggregator needs to know which number out of N it is. This is
        its bucket number.
    bucket_size : int, in seconds
        The amount of time each time bucket represents.
    lag : int, seconds, optional
        The number of seconds after a bucket's time ends to wait
        before processing the bucket. Default is 5s.
    """

    _time = time.time  # hook for faking time in tests

    def _ts_key(self, time):
        return int(time) / self.bucket_size

    @inlineCallbacks
    def startWorker(self):
        log.msg("Starting a MetricAggregator with config: %s" % self.config)
        bucket = int(self.config.get("bucket"))
        log.msg("MetricAggregator bucket %d" % bucket)
        self.bucket_size = int(self.config.get("bucket_size"))
        log.msg("Bucket size is %d seconds" % self.bucket_size)
        self.lag = float(self.config.get("lag", 5.0))

        # ts_key -> { metric_name -> (aggregate_set, values) }
        # values is a list of (timestamp, value) pairs
        self.buckets = {}
        # initialize last processed bucket
        self._last_ts_key = self._ts_key(self._time() - self.lag) - 2

        self.publisher = yield self.start_publisher(AggregatedMetricPublisher)
        self.consumer = yield self.start_consumer(TimeBucketConsumer,
                                                  bucket, self.consume_metric)

        self._task = LoopingCall(self.check_buckets)
        done = self._task.start(self.bucket_size, False)
        done.addErrback(lambda failure: log.err(failure,
                        "MetricAggregator bucket checking task died"))

    def check_buckets(self):
        """Periodically clean out old buckets and calculate aggregates."""
        # key for previous bucket
        current_ts_key = self._ts_key(self._time() - self.lag) - 1
        for ts_key in self.buckets.keys():
            if ts_key <= self._last_ts_key:
                log.err(DiscardedMetricError("Throwing way old metric data: %r"
                                             % self.buckets[ts_key]))
                del self.buckets[ts_key]
            elif ts_key <= current_ts_key:
                aggregates = []
                ts = ts_key * self.bucket_size
                items = self.buckets[ts_key].iteritems()
                for metric_name, (agg_set, values) in items:
                    values = [v for t, v in sorted(values)]
                    for agg_name in agg_set:
                        agg_metric = "%s.%s" % (metric_name, agg_name)
                        agg_func = Aggregator.from_name(agg_name)
                        agg_value = agg_func(values)
                        aggregates.append((agg_metric, agg_value))

                for agg_metric, agg_value in aggregates:
                    self.publisher.publish_aggregate(agg_metric, ts,
                                                     agg_value)
                del self.buckets[ts_key]
        self._last_ts_key = current_ts_key

    def consume_metric(self, metric_name, aggregates, values):
        if not values:
            return
        ts_key = self._ts_key(values[0][0])
        metrics = self.buckets.get(ts_key, None)
        if metrics is None:
            metrics = self.buckets[ts_key] = {}
        metric = metrics.get(metric_name)
        if metric is None:
            metric = metrics[metric_name] = (set(), [])
        existing_aggregates, existing_values = metric
        existing_aggregates.update(aggregates)
        existing_values.extend(values)

    def stopWorker(self):
        self._task.stop()
        self.check_buckets()


class MetricsCollectorWorker(Worker):
    @inlineCallbacks
    def startWorker(self):
        log.msg("Starting %s with config: %s" % (
                type(self).__name__, self.config))
        yield self.setup_worker()
        self.consumer = yield self.start_consumer(
            AggregatedMetricConsumer, self.consume_metrics)

    def stopWorker(self):
        log.msg("Stopping %s" % (type(self).__name__,))
        return self.teardown_worker()

    def setup_worker(self):
        pass

    def teardown_worker(self):
        pass

    def consume_metrics(self, metric_name, values):
        raise NotImplementedError()


class GraphitePublisher(Publisher):
    """Publisher for sending messages to Graphite."""

    exchange_name = "graphite"
    exchange_type = "topic"
    durable = True
    auto_delete = False
    delivery_mode = 2

    def publish_metric(self, metric, value, timestamp):
        self.publish_raw("%f %d" % (value, timestamp), routing_key=metric)


class GraphiteMetricsCollector(MetricsCollectorWorker):
    """Worker that collects Vumi metrics and publishes them to Graphite."""

    @inlineCallbacks
    def setup_worker(self):
        self.graphite_publisher = yield self.start_publisher(GraphitePublisher)

    def consume_metrics(self, metric_name, values):
        for timestamp, value in values:
            self.graphite_publisher.publish_metric(
                metric_name, value, timestamp)


class UDPMetricsProtocol(DatagramProtocol):
    def __init__(self, ip, port):
        # NOTE: `host` must be an IP, not a hostname.
        self._ip = ip
        self._port = port

    def startProtocol(self):
        self.transport.connect(self._ip, self._port)

    def send_metric(self, metric_string):
        return self.transport.write(metric_string)


class UDPMetricsCollector(MetricsCollectorWorker):
    """Worker that collects Vumi metrics and publishes them over UDP."""

    DEFAULT_FORMAT_STRING = '%(timestamp)s %(metric_name)s %(value)s\n'
    DEFAULT_TIMESTAMP_FORMAT = '%Y-%m-%d %H:%M:%S%z'

    @inlineCallbacks
    def setup_worker(self):
        self.format_string = self.config.get(
            'format_string', self.DEFAULT_FORMAT_STRING)
        self.timestamp_format = self.config.get(
            'timestamp_format', self.DEFAULT_TIMESTAMP_FORMAT)
        self.metrics_ip = yield reactor.resolve(self.config['metrics_host'])
        self.metrics_port = int(self.config['metrics_port'])
        self.metrics_protocol = UDPMetricsProtocol(
            self.metrics_ip, self.metrics_port)
        self.listener = yield reactor.listenUDP(0, self.metrics_protocol)

    def teardown_worker(self):
        return self.listener.stopListening()

    def consume_metrics(self, metric_name, values):
        for timestamp, value in values:
            timestamp = datetime.utcfromtimestamp(timestamp)
            metric_string = self.format_string % {
                'timestamp': timestamp.strftime(self.timestamp_format),
                'metric_name': metric_name,
                'value': value,
                }
            self.metrics_protocol.send_metric(metric_string)


class RandomMetricsGenerator(Worker):
    """Worker that publishes a set of random metrics.

    Useful for tests and demonstrations.

    Configuration Values
    --------------------
    manager_period : float in seconds, optional
        How often to have the internal metric manager send metrics
        messages. Default is 5s.
    generator_period: float in seconds, optional
        How often the random metric loop should send values to the
        metric manager. Default is 1s.
    """
    # callback for tests, f(worker)
    # (or anyone else that wants to be notified when metrics are generated)
    on_run = None

    @inlineCallbacks
    def startWorker(self):
        log.msg("Starting the MetricsGenerator with config: %s" % self.config)
        manager_period = float(self.config.get("manager_period", 5.0))
        log.msg("MetricManager will sent metrics every %s seconds" %
                manager_period)
        generator_period = float(self.config.get("generator_period", 1.0))
        log.msg("Random metrics values will be generated every %s seconds" %
                generator_period)

        self.mm = yield self.start_publisher(MetricManager, "vumi.random.",
                                             manager_period)
        self.counter = self.mm.register(Count("count"))
        self.value = self.mm.register(Metric("value"))
        self.timer = self.mm.register(Timer("timer"))
        self.next = Deferred()
        self.task = LoopingCall(self.run)
        self.task.start(generator_period)

    @inlineCallbacks
    def run(self):
        if random.choice([True, False]):
            self.counter.inc()
        self.value.set(random.normalvariate(2.0, 0.1))
        with self.timer.timeit():
            d = Deferred()
            wait = random.uniform(0.0, 0.1)
            reactor.callLater(wait, lambda: d.callback(None))
            yield d
        if self.on_run is not None:
            self.on_run(self)

    def stopWorker(self):
        self.mm.stop()
        self.task.stop()
        log.msg("Stopping the MetricsGenerator")



# -*- coding: utf-8 -*-

import os

from twisted.internet import reactor
from twisted.internet.defer import inlineCallbacks, returnValue, Deferred

from vumi.persist.fake_redis import FakeRedis, ResponseError
from vumi.tests.helpers import VumiTestCase


class FakeRedisTestMixin(object):
    """
    Test methods (and some unimplemented stubs) for FakeRedis.
    """

    def get_redis(self, **kwargs):
        """
        Return a Redis object (or a wrapper around one).
        """
        raise NotImplementedError(".get_redis() method not implemented.")

    def assert_redis_op(self, redis, expected, op, *args, **kw):
        """
        Assert that a redis operation returns the expected result.
        """
        raise NotImplementedError(".assert_redis_op() method not implemented.")

    def assert_redis_error(self, redis, op, *args, **kw):
        """
        Assert that a redis operation raises an exception.
        """
        raise NotImplementedError(
            ".assert_redis_error() method not implemented.")

    def wait(self, delay):
        """
        Wait some number of seconds, either for real or by advancing a clock.
        """
        raise NotImplementedError(".wait() method not implemented.")

    @inlineCallbacks
    def test_delete(self):
        redis = yield self.get_redis()
        yield redis.set("delete_me", 1)
        yield self.assert_redis_op(redis, True, 'delete', "delete_me")
        yield self.assert_redis_op(redis, False, 'delete', "delete_me")

    @inlineCallbacks
    def test_incr(self):
        redis = yield self.get_redis()
        yield redis.set("inc", 1)
        yield self.assert_redis_op(redis, '1', 'get', "inc")
        yield self.assert_redis_op(redis, 2, 'incr', "inc")
        yield self.assert_redis_op(redis, 3, 'incr', "inc")
        yield self.assert_redis_op(redis, '3', 'get', "inc")

    @inlineCallbacks
    def test_incrby(self):
        redis = yield self.get_redis()
        yield redis.set("inc", 1)
        yield self.assert_redis_op(redis, '1', 'get', "inc")
        yield self.assert_redis_op(redis, 3, 'incr', "inc", 2)
        yield self.assert_redis_op(redis, '3', 'get', "inc")

    @inlineCallbacks
    def test_decr(self):
        redis = yield self.get_redis()
        yield redis.set("dec", 4)
        yield self.assert_redis_op(redis, '4', 'get', "dec")
        yield self.assert_redis_op(redis, 3, 'decr', "dec")
        yield self.assert_redis_op(redis, 2, 'decr', "dec")
        yield self.assert_redis_op(redis, '2', 'get', "dec")

    @inlineCallbacks
    def test_decrby(self):
        redis = yield self.get_redis()
        yield redis.set("dec", 4)
        yield self.assert_redis_op(redis, '4', 'get', "dec")
        yield self.assert_redis_op(redis, 2, 'decr', "dec", 2)
        yield self.assert_redis_op(redis, '2', 'get', "dec")

    @inlineCallbacks
    def test_setnx(self):
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, False, 'exists', "mykey")
        yield self.assert_redis_op(redis, True, 'setnx', "mykey", "value")
        yield self.assert_redis_op(redis, "value", 'get', "mykey")
        yield self.assert_redis_op(redis, False, 'setnx', "mykey", "other")
        yield self.assert_redis_op(redis, "value", 'get', "mykey")

    @inlineCallbacks
    def test_setex(self):
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, False, 'exists', "mykey")
        yield self.assert_redis_op(redis, True, 'setex', "mykey", 10, "value")
        yield self.assert_redis_op(redis, "value", 'get', "mykey")
        yield self.assert_redis_op(redis, 10, 'ttl', "mykey")

    @inlineCallbacks
    def test_incr_with_by_param(self):
        redis = yield self.get_redis()
        yield redis.set("inc", 1)
        yield self.assert_redis_op(redis, '1', 'get', "inc")
        yield self.assert_redis_op(redis, 2, 'incr', "inc", 1)
        yield self.assert_redis_op(redis, 4, 'incr', "inc", 2)
        yield self.assert_redis_op(redis, 7, 'incr', "inc", 3)
        yield self.assert_redis_op(redis, 11, 'incr', "inc", 4)
        yield self.assert_redis_op(redis, 111, 'incr', "inc", 100)
        yield self.assert_redis_op(redis, '111', 'get', "inc")

    @inlineCallbacks
    def test_zadd(self):
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, 1, 'zadd', 'set', one=1.0)
        yield self.assert_redis_op(redis, 0, 'zadd', 'set', one=2.0)
        yield self.assert_redis_op(
            redis, [('one', 2.0)], 'zrange', 'set', 0, -1, withscores=True)
        yield self.assert_redis_error(redis, "zadd", "set", one='foo')
        yield self.assert_redis_error(redis, "zadd", "set", one=None)

    @inlineCallbacks
    def test_zrange(self):
        redis = yield self.get_redis()
        yield redis.zadd('set', one=0.1, two=0.2, three=0.3)
        yield self.assert_redis_op(redis, ['one'], 'zrange', 'set', 0, 0)
        yield self.assert_redis_op(
            redis, ['one', 'two'], 'zrange', 'set', 0, 1)
        yield self.assert_redis_op(
            redis, ['one', 'two', 'three'], 'zrange', 'set', 0, 2)
        yield self.assert_redis_op(
            redis, ['one', 'two', 'three'], 'zrange', 'set', 0, 3)
        yield self.assert_redis_op(
            redis, ['one', 'two', 'three'], 'zrange', 'set', 0, -1)
        yield self.assert_redis_op(
            redis, [('one', 0.1), ('two', 0.2), ('three', 0.3)],
            'zrange', 'set', 0, -1, withscores=True)
        yield self.assert_redis_op(
            redis, ['three', 'two', 'one'], 'zrange', 'set', 0, -1, desc=True)
        yield self.assert_redis_op(
            redis, [('three', 0.3), ('two', 0.2), ('one', 0.1)],
            'zrange', 'set', 0, -1, withscores=True, desc=True)
        yield self.assert_redis_op(
            redis, [('three', 0.3)],
            'zrange', 'set', 0, 0, withscores=True, desc=True)

    @inlineCallbacks
    def test_zrangebyscore(self):
        redis = yield self.get_redis()
        yield redis.zadd(
            'set', one=0.1, two=0.2, three=0.3, four=0.4, five=0.5)
        yield self.assert_redis_op(
            redis, ['two', 'three', 'four'], 'zrangebyscore', 'set', 0.2, 0.4)
        yield self.assert_redis_op(
            redis, ['two', 'three'], 'zrangebyscore', 'set', 0.2, 0.4, 0, 2)
        yield self.assert_redis_op(
            redis, ['three'], 'zrangebyscore', 'set', '(0.2', '(0.4')
        yield self.assert_redis_op(
            redis, ['two', 'three', 'four', 'five'],
            'zrangebyscore', 'set', '0.2', '+inf')
        yield self.assert_redis_op(
            redis, ['one', 'two'], 'zrangebyscore', 'set', '-inf', '0.2')

    @inlineCallbacks
    def test_zcount(self):
        redis = yield self.get_redis()
        yield redis.zadd(
            'set', one=0.1, two=0.2, three=0.3, four=0.4, five=0.5)
        yield self.assert_redis_op(redis, 3, 'zcount', 'set', 0.2, 0.4)

    @inlineCallbacks
    def test_zrangebyscore_with_scores(self):
        redis = yield self.get_redis()
        yield redis.zadd(
            'set', one=0.1, two=0.2, three=0.3, four=0.4, five=0.5)
        yield self.assert_redis_op(
            redis, [('two', 0.2), ('three', 0.3), ('four', 0.4)],
            'zrangebyscore', 'set', 0.2, 0.4, withscores=True)

    @inlineCallbacks
    def test_zcard(self):
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, 0, 'zcard', 'set')
        yield redis.zadd('set', one=0.1, two=0.2)
        yield self.assert_redis_op(redis, 2, 'zcard', 'set')
        yield redis.zadd('set', three=0.3)
        yield self.assert_redis_op(redis, 3, 'zcard', 'set')

    @inlineCallbacks
    def test_zrem(self):
        redis = yield self.get_redis()
        yield redis.zadd('set', one=0.1, two=0.2)
        yield self.assert_redis_op(redis, True, 'zrem', 'set', 'one')
        yield self.assert_redis_op(redis, False, 'zrem', 'set', 'one')
        yield self.assert_redis_op(
            redis, [('two', 0.2)], 'zrange', 'set', 0, -1, withscores=True)

    @inlineCallbacks
    def test_zremrangebyrank(self):
        redis = yield self.get_redis()
        yield redis.zadd('set', one=1, two=2, three=3)
        yield self.assert_redis_op(redis, 2, 'zremrangebyrank', 'set', 0, 1)
        yield self.assert_redis_op(
            redis, [('three', 3)], 'zrange', 'set', 0, -1, withscores=True)

    @inlineCallbacks
    def test_zremrangebyrank_empty_range(self):
        redis = yield self.get_redis()
        yield redis.zadd('set', one=1, two=2, three=3)
        yield self.assert_redis_op(redis, 0, 'zremrangebyrank', 'set', 10, 11)
        yield self.assert_redis_op(
            redis, [('one', 1), ('two', 2), ('three', 3)],
            'zrange', 'set', 0, -1, withscores=True)

    @inlineCallbacks
    def test_zremrangebyrank_negative_empty_range(self):
        redis = yield self.get_redis()
        yield redis.zadd('set', one=1, two=2, three=3)
        yield self.assert_redis_op(
            redis, 0, 'zremrangebyrank', 'set', -11, -10)
        yield self.assert_redis_op(
            redis, [('one', 1), ('two', 2), ('three', 3)],
            'zrange', 'set', 0, -1, withscores=True)

    @inlineCallbacks
    def test_zremrangebyrank_negative_start(self):
        redis = yield self.get_redis()
        yield redis.zadd('set', one=1, two=2, three=3)
        yield self.assert_redis_op(redis, 2, 'zremrangebyrank', 'set', -2, 2)
        yield self.assert_redis_op(
            redis, [('one', 1)], 'zrange', 'set', 0, -1, withscores=True)

    @inlineCallbacks
    def test_zremrangebyrank_negative_start_empty_range(self):
        redis = yield self.get_redis()
        yield redis.zadd('set', one=1, two=2, three=3)
        yield self.assert_redis_op(redis, 0, 'zremrangebyrank', 'set', -1, 1)
        yield self.assert_redis_op(
            redis, [('one', 1), ('two', 2), ('three', 3)],
            'zrange', 'set', 0, -1, withscores=True)

    @inlineCallbacks
    def test_zremrangebyrank_negative_stop(self):
        redis = yield self.get_redis()
        yield redis.zadd('set', one=1, two=2, three=3)
        yield self.assert_redis_op(redis, 2, 'zremrangebyrank', 'set', 1, -1)
        yield self.assert_redis_op(
            redis, [('one', 1)], 'zrange', 'set', 0, -1, withscores=True)

    @inlineCallbacks
    def test_zremrangebyrank_negative_stop_empty_range(self):
        redis = yield self.get_redis()
        yield redis.zadd('set', one=1, two=2, three=3)
        yield self.assert_redis_op(redis, 0, 'zremrangebyrank', 'set', 0, -5)
        yield self.assert_redis_op(
            redis, [('one', 1), ('two', 2), ('three', 3)],
            'zrange', 'set', 0, -1, withscores=True)

    @inlineCallbacks
    def test_zscore(self):
        redis = yield self.get_redis()
        yield redis.zadd('set', one=0.1, two=0.2)
        yield self.assert_redis_op(redis, 0.1, 'zscore', 'set', 'one')
        yield self.assert_redis_op(redis, 0.2, 'zscore', 'set', 'two')

    @inlineCallbacks
    def test_hgetall_returns_copy(self):
        redis = yield self.get_redis()
        yield redis.hset("hash", "foo", "1")
        data = yield redis.hgetall("hash")
        data["foo"] = "2"
        yield self.assert_redis_op(redis, {"foo": "1"}, 'hgetall', "hash")

    @inlineCallbacks
    def test_hincrby(self):
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, 1, 'hincrby', "inc", "field1")
        yield self.assert_redis_op(redis, 2, 'hincrby', "inc", "field1")
        yield self.assert_redis_op(redis, 5, 'hincrby', "inc", "field1", 3)
        yield self.assert_redis_op(redis, 7, 'hincrby', "inc", "field1", "2")
        yield self.assert_redis_error(redis, "hincrby", "inc", "field1", "1.5")
        yield redis.hset("inc", "field2", "a")
        yield self.assert_redis_error(redis, "hincrby", "inc", "field2")
        yield redis.set("key", "string")
        yield self.assert_redis_error(redis, "hincrby", "key", "field1")

    @inlineCallbacks
    def test_hexists(self):
        redis = yield self.get_redis()
        yield redis.hset('key', 'field', 1)
        yield self.assert_redis_op(redis, True, 'hexists', 'key', 'field')
        yield redis.hdel('key', 'field')
        yield self.assert_redis_op(redis, False, 'hexists', 'key', 'field')

    @inlineCallbacks
    def test_hsetnx(self):
        redis = yield self.get_redis()
        yield redis.hset('key', 'field', 1)
        self.assert_redis_op(redis, 0, 'hsetnx', 'key', 'field', 2)
        self.assertEqual((yield redis.hget('key', 'field')), '1')
        self.assert_redis_op(redis, 1, 'hsetnx', 'key', 'other-field', 2)
        self.assertEqual((yield redis.hget('key', 'other-field')), '2')

    @inlineCallbacks
    def test_sadd(self):
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, 1, 'sadd', 'set', 1)
        yield self.assert_redis_op(redis, 3, 'sadd', 'set', 2, 3, 4)
        yield self.assert_redis_op(
            redis, set(['1', '2', '3', '4']), 'smembers', 'set')

    @inlineCallbacks
    def test_smove(self):
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, 1, 'sadd', 'set1', 1)
        yield self.assert_redis_op(redis, 1, 'sadd', 'set2', 2)
        yield self.assert_redis_op(redis, True, 'smove', 'set1', 'set2', '1')
        yield self.assert_redis_op(redis, set(), 'smembers', 'set1')
        yield self.assert_redis_op(redis, set(['1', '2']), 'smembers', 'set2')

        yield self.assert_redis_op(redis, False, 'smove', 'set1', 'set2', '1')
        yield self.assert_redis_op(redis, True, 'smove', 'set2', 'set3', '1')
        yield self.assert_redis_op(redis, set(['2']), 'smembers', 'set2')
        yield self.assert_redis_op(redis, set(['1']), 'smembers', 'set3')

    @inlineCallbacks
    def test_sunion(self):
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, 1, 'sadd', 'set1', 1)
        yield self.assert_redis_op(redis, 1, 'sadd', 'set2', 2)
        yield self.assert_redis_op(redis, set(['1']), 'sunion', 'set1')
        yield self.assert_redis_op(
            redis, set(['1', '2']), 'sunion', 'set1', 'set2')
        yield self.assert_redis_op(redis, set(), 'sunion', 'other')

    @inlineCallbacks
    def test_lpush(self):
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, 1, 'lpush', 'list', 1)
        yield self.assert_redis_op(redis, ['1'], 'lrange', 'list', 0, -1)
        yield self.assert_redis_op(redis, 2, 'lpush', 'list', 'a')
        yield self.assert_redis_op(redis, ['a', '1'], 'lrange', 'list', 0, -1)
        yield self.assert_redis_op(redis, 3, 'lpush', 'list', '7')
        yield self.assert_redis_op(
            redis, ['7', 'a', '1'], 'lrange', 'list', 0, -1)

    @inlineCallbacks
    def test_rpush(self):
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, 1, 'rpush', 'list', 1)
        yield self.assert_redis_op(redis, ['1'], 'lrange', 'list', 0, -1)
        yield self.assert_redis_op(redis, 2, 'rpush', 'list', 'a')
        yield self.assert_redis_op(redis, ['1', 'a'], 'lrange', 'list', 0, -1)
        yield self.assert_redis_op(redis, 3, 'rpush', 'list', '7')
        yield self.assert_redis_op(
            redis, ['1', 'a', '7'], 'lrange', 'list', 0, -1)

    @inlineCallbacks
    def test_rpop(self):
        redis = yield self.get_redis()
        yield redis.lpush('key', 1)
        yield redis.lpush('key', 'a')
        yield redis.lpush('key', '3')
        yield self.assert_redis_op(redis, '1', 'rpop', 'key')
        yield self.assert_redis_op(redis, 'a', 'rpop', 'key')
        yield self.assert_redis_op(redis, '3', 'rpop', 'key')
        yield self.assert_redis_op(redis, None, 'rpop', 'key')

    @inlineCallbacks
    def test_rpoplpush(self):
        redis = yield self.get_redis()
        yield redis.lpush('source', 1)
        yield redis.lpush('source', 'a')
        yield redis.lpush('source', '3')
        yield self.assert_redis_op(
            redis, '1', 'rpoplpush', 'source', 'destination')
        yield self.assert_redis_op(
            redis, 'a', 'rpoplpush', 'source', 'destination')
        yield self.assert_redis_op(
            redis, '3', 'rpoplpush', 'source', 'destination')
        yield self.assert_redis_op(redis, None, 'rpop', 'source')
        yield self.assert_redis_op(redis, '1', 'rpop', 'destination')
        yield self.assert_redis_op(redis, 'a', 'rpop', 'destination')
        yield self.assert_redis_op(redis, '3', 'rpop', 'destination')
        yield self.assert_redis_op(redis, None, 'rpop', 'destination')

    @inlineCallbacks
    def test_lrem(self):
        redis = yield self.get_redis()
        for i in range(5):
            yield self.assert_redis_op(
                redis, 2 * i + 1, 'rpush', 'list', 'v%d' % i)
            yield self.assert_redis_op(redis, 2 * i + 2, 'rpush', 'list', 1)
        yield self.assert_redis_op(redis, 5, 'lrem', 'list', 1)
        yield self.assert_redis_op(
            redis, ['v0', 'v1', 'v2', 'v3', 'v4'], 'lrange', 'list', 0, -1)

    @inlineCallbacks
    def test_lrem_positive_num(self):
        redis = yield self.get_redis()
        for i in range(5):
            yield self.assert_redis_op(
                redis, 2 * i + 1, 'rpush', 'list', 'v%d' % i)
            yield self.assert_redis_op(redis, 2 * i + 2, 'rpush', 'list', 1)
        yield self.assert_redis_op(redis, 2, 'lrem', 'list', 1, 2)
        yield self.assert_redis_op(
            redis, ['v0', 'v1', 'v2', '1', 'v3', '1', 'v4', '1'],
            'lrange', 'list', 0, -1)

    @inlineCallbacks
    def test_lrem_negative_num(self):
        redis = yield self.get_redis()
        for i in range(5):
            yield self.assert_redis_op(
                redis, 2 * i + 1, 'rpush', 'list', 'v%d' % i)
            yield self.assert_redis_op(redis, 2 * i + 2, 'rpush', 'list', 1)
        yield self.assert_redis_op(redis, 2, 'lrem', 'list', 1, -2)
        yield self.assert_redis_op(
            redis, ['v0', '1', 'v1', '1', 'v2', '1', 'v3', 'v4'],
            'lrange', 'list', 0, -1)

    @inlineCallbacks
    def test_ltrim(self):
        redis = yield self.get_redis()
        for i in range(1, 5):
            yield self.assert_redis_op(redis, i, 'rpush', 'list', str(i))
        yield self.assert_redis_op(
            redis, ['1', '2', '3', '4'], 'lrange', 'list', 0, -1)
        yield self.assert_redis_op(redis, True, 'ltrim', 'list', 1, 2)
        yield self.assert_redis_op(redis, ['2', '3'], 'lrange', 'list', 0, -1)

    @inlineCallbacks
    def test_ltrim_mid_range(self):
        redis = yield self.get_redis()
        for i in range(1, 6):
            yield self.assert_redis_op(redis, i, 'rpush', 'list', str(i))
        yield self.assert_redis_op(
            redis, ['1', '2', '3', '4', '5'], 'lrange', 'list', 0, -1)
        yield self.assert_redis_op(redis, True, 'ltrim', 'list', 2, 3)
        yield self.assert_redis_op(redis, ['3', '4'], 'lrange', 'list', 0, -1)

    @inlineCallbacks
    def test_ltrim_keep_all(self):
        redis = yield self.get_redis()
        for i in range(1, 4):
            yield self.assert_redis_op(redis, i, 'rpush', 'list', str(i))
        yield self.assert_redis_op(
            redis, ['1', '2', '3'], 'lrange', 'list', 0, -1)
        yield self.assert_redis_op(redis, True, 'ltrim', 'list', 0, -1)
        yield self.assert_redis_op(
            redis, ['1', '2', '3'], 'lrange', 'list', 0, -1)

    @inlineCallbacks
    def test_expire_persist_ttl(self):
        redis = yield self.get_redis()
        # Missing key.
        yield self.assert_redis_op(redis, None, 'ttl', "tempval")
        yield self.assert_redis_op(redis, 0, 'expire', "tempval", 10)
        yield self.assert_redis_op(redis, 0, 'persist', "tempval")
        # Persistent key.
        yield redis.set("tempval", 1)
        yield self.assert_redis_op(redis, None, 'ttl', "tempval")
        yield self.assert_redis_op(redis, 0, 'persist', "tempval")
        yield self.assert_redis_op(redis, 1, 'expire', "tempval", 10)
        # Temporary key.
        yield self.assert_redis_op(redis, 10, 'ttl', "tempval")
        yield self.wait(redis, 0.6)  # Wait a bit for the TTL to change.
        yield self.assert_redis_op(redis, 9, 'ttl', "tempval")
        yield self.assert_redis_op(redis, 1, 'expire', "tempval", 5)
        yield self.assert_redis_op(redis, 5, 'ttl', "tempval")
        yield self.assert_redis_op(redis, 1, 'persist', "tempval")
        # Persistent key again.
        yield redis.set("tempval", 1)
        yield self.assert_redis_op(redis, None, 'ttl', "tempval")
        yield self.assert_redis_op(redis, 0, 'persist', "tempval")
        yield self.assert_redis_op(redis, 1, 'expire', "tempval", 10)

    @inlineCallbacks
    def test_type(self):
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, 'none', 'type', 'unknown_key')
        yield redis.set("string_key", "a")
        yield self.assert_redis_op(redis, 'string', 'type', 'string_key')
        yield redis.lpush("list_key", "a")
        yield self.assert_redis_op(redis, 'list', 'type', 'list_key')
        yield redis.sadd("set_key", "a")
        yield self.assert_redis_op(redis, 'set', 'type', 'set_key')
        yield redis.zadd("zset_key", a=1.0)
        yield self.assert_redis_op(redis, 'zset', 'type', 'zset_key')
        yield redis.hset("hash_key", "a", 1.0)
        yield self.assert_redis_op(redis, 'hash', 'type', 'hash_key')

    @inlineCallbacks
    def test_charset_encoding_default(self):
        # Redis client assumes utf-8
        redis = yield self.get_redis()
        yield redis.set('name', u'Zoë Destroyer of Ascii')
        yield self.assert_redis_op(
            redis, 'Zo\xc3\xab Destroyer of Ascii', 'get', 'name')

    @inlineCallbacks
    def test_charset_encoding_custom_replace(self):
        redis = yield self.get_redis(charset='ascii', errors='replace')
        yield redis.set('name', u'Zoë Destroyer of Ascii')
        yield self.assert_redis_op(
            redis, 'Zo? Destroyer of Ascii', 'get', 'name')

    @inlineCallbacks
    def test_charset_encoding_custom_ignore(self):
        redis = yield self.get_redis(charset='ascii', errors='ignore')
        yield redis.set('name', u'Zoë Destroyer of Ascii')
        yield self.assert_redis_op(
            redis, 'Zo Destroyer of Ascii', 'get', 'name')

    @inlineCallbacks
    def test_scan_no_keys(self):
        """
        Real and fake Redis implementation all return the same (empty) response
        when we scan for keys that don't exist. Other scanning methods are in
        FakeRedisUnverifiedTestMixin because we can't fake the same arbitrary
        order in which keys are returned from real Redis.
        """
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, [None, []], 'scan', None)

    @inlineCallbacks
    def test_pfadd_and_pfcount(self):
        """
        We can't test these two things separately, so test them together.
        """
        redis = yield self.get_redis()
        yield self.assert_redis_op(redis, 1, 'pfadd', 'hll1', 'a')
        yield self.assert_redis_op(redis, 1, 'pfcount', 'hll1')
        yield self.assert_redis_op(redis, 0, 'pfadd', 'hll1', 'a')
        yield self.assert_redis_op(redis, 1, 'pfcount', 'hll1')

        yield self.assert_redis_op(redis, 1, 'pfadd', 'hll2', 'a', 'b')
        yield self.assert_redis_op(redis, 2, 'pfcount', 'hll2')
        yield self.assert_redis_op(redis, 0, 'pfadd', 'hll2', 'a', 'b')
        yield self.assert_redis_op(redis, 2, 'pfcount', 'hll2')


class FakeRedisUnverifiedTestMixin(object):
    """
    This mixin adds some extra tests that are not verified against real Redis.

    Each test in here should explain why verification isn't possible.
    """

    @inlineCallbacks
    def test_scan_simple(self):
        """
        Scanning returns keys in an order that depends on arbitrary state in
        the Redis server, so we can't fake it in a way that's identical to real
        Redis.
        """
        redis = yield self.get_redis()
        for i in range(20):
            yield redis.set("key%02d" % i, str(i))
        # Ordered the way FakeRedis.scan() returns them.
        result_keys = redis._sort_keys_by_hash(
            ["key%02d" % i for i in range(20)])

        self.assert_redis_op(redis, ['10', result_keys[:10]], 'scan', None)
        self.assert_redis_op(
            redis, ['5', result_keys[:5]], 'scan', None, count=5)
        self.assert_redis_op(
            redis, ['10', result_keys[5:10]], 'scan', '5', count=5)
        self.assert_redis_op(
            redis, [None, result_keys[15:]], 'scan', '15', count=5)
        self.assert_redis_op(
            redis, [None, result_keys], 'scan', None, count=20)

    @inlineCallbacks
    def test_scan_interleaved_key_changes(self):
        """
        Scanning returns keys in an order that depends on arbitrary state in
        the Redis server, so we can't fake it in a way that's identical to real
        Redis.
        """
        redis = yield self.get_redis()
        for i in range(20):
            yield redis.set("key%02d" % i, str(i))
        # Ordered the way FakeRedis.scan() returns them.
        result_keys = redis._sort_keys_by_hash(
            ["key%02d" % i for i in range(20)])

        self.assert_redis_op(redis, ['10', result_keys[:10]], 'scan', None)

        # Set and delete a bunch of keys to change some internal state. The
        # next call to scan() will return duplicates.
        for i in range(20):
            yield redis.set("transient%02d" % i, str(i))
            yield redis.delete("transient%02d" % i)

        self.assert_redis_op(redis, ['31', result_keys[5:15]], 'scan', '10')
        self.assert_redis_op(redis, [None, result_keys[15:]], 'scan', '31')

    @inlineCallbacks
    def test_pfadd_and_pfcount_large(self):
        """
        for large sets, we get approximate counts. Redis and hyperloglog use
        different hash functions, so we get different approximations out of
        them and can't verify the results.
        """
        redis = yield self.get_redis()
        values = ['v%s' % i for i in xrange(1000)]
        yield self.assert_redis_op(redis, 1, 'pfadd', 'hll1', *values)
        yield self.assert_redis_op(redis, 998, 'pfcount', 'hll1')
        yield self.assert_redis_op(redis, 0, 'pfadd', 'hll1', *values)
        yield self.assert_redis_op(redis, 998, 'pfcount', 'hll1')


class TestFakeRedis(FakeRedisUnverifiedTestMixin, FakeRedisTestMixin,
                    VumiTestCase):

    def get_redis(self, **kwargs):
        redis = FakeRedis(**kwargs)
        self.add_cleanup(redis.teardown)
        return redis

    def assert_redis_op(self, redis, expected, op, *args, **kw):
        self.assertEqual(expected, getattr(redis, op)(*args, **kw))

    def assert_redis_error(self, redis, op, *args, **kw):
        self.assertRaises(Exception, getattr(redis, op), *args, **kw)

    def wait(self, redis, delay):
        redis.clock.advance(delay)


class TestFakeRedisAsync(FakeRedisUnverifiedTestMixin, FakeRedisTestMixin,
                         VumiTestCase):

    def get_redis(self, **kwargs):
        redis = FakeRedis(async=True, **kwargs)
        self.add_cleanup(redis.teardown)
        return redis

    def assert_redis_op(self, redis, expected, op, *args, **kw):
        d = getattr(redis, op)(*args, **kw)
        return d.addCallback(lambda r: self.assertEqual(expected, r))

    def assert_redis_error(self, redis, op, *args, **kw):
        d = getattr(redis, op)(*args, **kw)
        return self.assertFailure(d, Exception)

    def wait(self, redis, delay):
        redis.clock.advance(delay)


class RedisPairWrapper(object):
    def __init__(self, test_case, fake_redis, real_redis):
        self._test_case = test_case
        self._fake_redis = fake_redis
        self._real_redis = real_redis
        self._perform_operation = self._real_redis.call_decorator(
            self._perform_operation_gen)

    def _perform_operation_gen(self, op, *args, **kw):
        """
        Perform an operation on both the fake and real Redises and assert that
        the responses and errors are the same.

        NOTE: This method is a generator and is not used directly. It's wrapped
              with an appropriate sync/async wrapper in __init__() above.
        """
        results = []
        errors = []
        for redis in [self._fake_redis, self._real_redis]:
            try:
                result = yield getattr(redis, op)(*args, **kw)
            except Exception as e:
                errors.append(e)
                if results != []:
                    self._test_case.fail(
                        "Fake redis returned %r but real redis raised %r" % (
                            results[0], errors[0]))
            else:
                results.append(result)
                if errors != []:
                    self._test_case.fail(
                        "Real redis returned %r but fake redis raised %r" % (
                            results[0], errors[0]))

        # First, handle errors.
        if errors:
            # We match based on the exception type name so that ResponseErrors
            # from different places are considered equal. We ignore the error
            # message in the check, but we display it to aid debugging.
            self._test_case.assertEqual(
                type(errors[0]).__name__, type(errors[1]).__name__,
                ("Fake redis (a) and real redis (b) errors different:"
                 "\n a = %r\n b = %r") % tuple(errors))
            raise errors[0]

        # Now handle results.
        self._test_case.assertEqual(
            results[0], results[1],
            "Fake redis (a) and real redis (b) responses different:")
        returnValue(results[0])

    def __getattr__(self, name):
        return lambda *args, **kw: self._perform_operation(name, *args, **kw)


class TestFakeRedisVerify(FakeRedisTestMixin, VumiTestCase):

    if 'VUMITEST_REDIS_DB' not in os.environ:
        skip = ("This test requires a real Redis server. Set VUMITEST_REDIS_DB"
                " to run it.")

    def get_redis(self, **kwargs):
        from vumi.persist.redis_manager import RedisManager
        # Fake redis
        fake_redis = FakeRedis(**kwargs)
        self.add_cleanup(fake_redis.teardown)
        # Real redis
        config = {
            'FAKE_REDIS': 'yes',
            'key_prefix': 'redistest',
        }
        config.update(kwargs)
        real_redis = RedisManager.from_config(config)
        self.add_cleanup(self.cleanup_manager, real_redis)
        real_redis._purge_all()
        # Both redises
        return RedisPairWrapper(self, fake_redis, real_redis)

    def cleanup_manager(self, manager):
        manager._purge_all()
        manager._close()

    def assert_redis_op(self, redis, expected, op, *args, **kw):
        self.assertEqual(expected, getattr(redis, op)(*args, **kw))

    def assert_redis_error(self, redis, op, *args, **kw):
        self.assertRaises(ResponseError, getattr(redis, op), *args, **kw)

    def wait(self, redis, delay):
        redis._fake_redis.clock.advance(delay)
        d = Deferred()
        reactor.callLater(delay, d.callback, None)
        return d


class TestFakeRedisVerifyAsync(FakeRedisTestMixin, VumiTestCase):

    if 'VUMITEST_REDIS_DB' not in os.environ:
        skip = ("This test requires a real Redis server. Set VUMITEST_REDIS_DB"
                " to run it.")

    @inlineCallbacks
    def get_redis(self, **kwargs):
        from vumi.persist.txredis_manager import TxRedisManager
        # Fake redis
        fake_redis = FakeRedis(async=True, **kwargs)
        self.add_cleanup(fake_redis.teardown)
        # Real redis
        config = {
            'FAKE_REDIS': 'yes',
            'key_prefix': 'redistest',
        }
        config.update(kwargs)
        real_redis = yield TxRedisManager.from_config(config)
        self.add_cleanup(self.cleanup_manager, real_redis)
        # Both redises
        yield real_redis._purge_all()

        returnValue(RedisPairWrapper(self, fake_redis, real_redis))

    @inlineCallbacks
    def cleanup_manager(self, manager):
        yield manager._purge_all()
        yield manager._close()

    def assert_redis_op(self, redis, expected, op, *args, **kw):
        d = getattr(redis, op)(*args, **kw)
        return d.addCallback(lambda r: self.assertEqual(expected, r))

    def assert_redis_error(self, redis, op, *args, **kw):
        d = getattr(redis, op)(*args, **kw)
        return self.assertFailure(d, ResponseError)

    def wait(self, redis, delay):
        redis._fake_redis.clock.advance(delay)
        d = Deferred()
        reactor.callLater(delay, d.callback, None)
        return d



# -*- test-case-name: vumi.tests.test_log -*-
import logging
from functools import partial

from twisted.python import log


debug = partial(log.msg, logLevel=logging.DEBUG)
info = partial(log.msg, logLevel=logging.INFO)
warning = partial(log.msg, logLevel=logging.WARNING)
error = partial(log.err, logLevel=logging.ERROR)
critical = partial(log.err, logLevel=logging.CRITICAL)

# make transition from twisted.python.log easier
msg = info
err = error



from setuptools import setup, find_packages


setup(
    name="vumi",
    version="0.5.18",
    url='http://github.com/praekelt/vumi',
    license='BSD',
    description="Super-scalable messaging engine for the delivery of SMS, "
                "Star Menu and chat messages to diverse audiences in "
                "emerging markets and beyond.",
    long_description=open('README.rst', 'r').read(),
    author='Praekelt Foundation',
    author_email='dev@praekeltfoundation.org',
    packages=find_packages() + [
        'twisted.plugins',
    ],
    package_data={'twisted.plugins': ['twisted/plugins/*.py']},
    include_package_data=True,
    scripts=[
        'vumi/scripts/vumi_tagpools.py',
        'vumi/scripts/vumi_redis_tools.py',
        'vumi/scripts/vumi_model_migrator.py',
        'vumi/scripts/vumi_count_models.py',
        'vumi/scripts/vumi_list_messages.py',
    ],
    install_requires=[
        'zope.interface',
        'Twisted>=13.1.0',
        'txAMQP>=0.6.2',
        'PyYAML',
        'iso8601',
        'pyOpenSSL',
        'service_identity',
        'txssmi>=0.3.0',
        'wokkel',
        'redis>=2.10.0',
        'txredis',
        'python-smpp>=0.1.5',
        'pytz',
        'riak>=2.1',
        'txJSON-RPC==0.3.1',
        'txTwitter>=0.1.4a',
        'treq',
        'confmodel>=0.2.0',
        'hyperloglog',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Networking',
    ],
)



# -*- test-case-name: vumi.transports.dmark.tests.test_dmark_ussd -*-

import json

from twisted.internet.defer import inlineCallbacks, returnValue
from twisted.web import http

from vumi import log
from vumi.components.session import SessionManager
from vumi.config import ConfigDict, ConfigInt
from vumi.message import TransportUserMessage
from vumi.transports.httprpc import HttpRpcTransport


class DmarkUssdTransportConfig(HttpRpcTransport.CONFIG_CLASS):
    """Config for Dmark USSD transport."""

    ussd_session_timeout = ConfigInt(
        "Number of seconds before USSD session information stored in Redis"
        " expires.",
        default=600, static=True)

    redis_manager = ConfigDict(
        "Redis client configuration.", default={}, static=True)


class DmarkUssdTransport(HttpRpcTransport):
    """Dmark USSD transport over HTTP.

    When a USSD message is received, Dmark will make an HTTP GET request to
    the transport with the following query parameters:

    * ``transactionId``: A unique ID for the USSD session (string).
    * ``msisdn``: The phone number that the message was sent from (string).
    * ``ussdServiceCode``: The USSD Service code the request was made to
      (string).
    * ``transactionTime``: The time the USSD request was received at Dmark,
      as a Unix timestamp (UTC).
    * ``ussdRequestString``: The full content of the USSD request(string).
    * ``creationTime``: The time the USSD request was sent, as a Unix
      timestamp (UTC), if available. (This time is given by the mobile
      network, and may not always be reliable.)
    * ``response``: ``"false"`` if this is a new session, ``"true"`` if it is
      not. Currently not used by the transport (it relies on the
      ``transactionId`` being unique instead).

    The transport may respond to this request either using JSON or form-encoded
    data. A successful response must return HTTP status code 200. Any other
    response code is treated as a failure.

    This transport responds with JSON encoded data. The JSON response
    contains the following keys:

    * ``responseString``: The content to be returned to the phone number that
      originated the USSD request.
    * ``action``: Either ``end`` or ``request``. ``end`` signifies that no
      further interaction is expected from the user and the USSD session should
      be closed. ``request`` signifies that further interaction is expected.

    **Example JSON response**:

    .. sourcecode: javascript

       {
         "responseString": "Hello from Vumi!",
         "action": "end"
       }
    """

    CONFIG_CLASS = DmarkUssdTransportConfig

    transport_type = 'ussd'

    ENCODING = 'utf-8'
    EXPECTED_FIELDS = frozenset([
        'transactionId', 'msisdn', 'ussdServiceCode', 'transactionTime',
        'ussdRequestString', 'creationTime', 'response',
    ])

    @inlineCallbacks
    def setup_transport(self):
        yield super(DmarkUssdTransport, self).setup_transport()
        config = self.get_static_config()
        r_prefix = "vumi.transports.dmark_ussd:%s" % self.transport_name
        self.session_manager = yield SessionManager.from_redis_config(
            config.redis_manager, r_prefix,
            max_session_length=config.ussd_session_timeout)

    @inlineCallbacks
    def teardown_transport(self):
        yield super(DmarkUssdTransport, self).teardown_transport()
        yield self.session_manager.stop()

    @inlineCallbacks
    def session_event_for_transaction(self, transaction_id):
        # XXX: There is currently no way to detect when the user closes
        #      the session (i.e. TransportUserMessage.SESSION_CLOSE)
        session_id = transaction_id
        session = yield self.session_manager.load_session(transaction_id)
        if session:
            session_event = TransportUserMessage.SESSION_RESUME
            yield self.session_manager.save_session(session_id, session)
        else:
            session_event = TransportUserMessage.SESSION_NEW
            yield self.session_manager.create_session(
                session_id, transaction_id=transaction_id)
        returnValue(session_event)

    @inlineCallbacks
    def handle_raw_inbound_message(self, request_id, request):
        values, errors = self.get_field_values(request, self.EXPECTED_FIELDS)
        if errors:
            log.msg('Unhappy incoming message: %r' % (errors,))
            self.finish_request(
                request_id, json.dumps(errors), code=http.BAD_REQUEST)
            return

        to_addr = values["ussdServiceCode"]
        from_addr = values["msisdn"]
        session_event = yield self.session_event_for_transaction(
            values["transactionId"])

        yield self.publish_message(
            message_id=request_id,
            content=values["ussdRequestString"],
            to_addr=to_addr,
            from_addr=from_addr,
            provider='dmark',
            session_event=session_event,
            transport_type=self.transport_type,
            transport_metadata={
                'dmark_ussd': {
                    'transaction_id': values['transactionId'],
                    'transaction_time': values['transactionTime'],
                    'creation_time': values['creationTime'],
                }
            })

    def handle_outbound_message(self, message):
        self.emit("DmarkUssdTransport consuming %r" % (message,))
        missing_fields = self.ensure_message_values(message,
                            ['in_reply_to', 'content'])
        if missing_fields:
            return self.reject_message(message, missing_fields)

        if message["session_event"] == TransportUserMessage.SESSION_CLOSE:
            action = "end"
        else:
            action = "request"

        response_data = {
            "responseString": message["content"],
            "action": action,
        }

        response_id = self.finish_request(
            message['in_reply_to'], json.dumps(response_data))
        if response_id is not None:
            return self.publish_ack(
                user_message_id=message['message_id'],
                sent_message_id=message['message_id'])
        else:
            return self.publish_nack(
                user_message_id=message['message_id'],
                sent_message_id=message['message_id'],
                reason="Could not find original request.")



# -*- coding: utf-8 -*-

"""Tests for vumi.persist.fields."""

from datetime import datetime

from vumi.message import Message

from vumi.persist.fields import (
    ValidationError, Field, Integer, Unicode, Tag, Timestamp, Json,
    ListOf, SetOf, Dynamic, FieldWithSubtype, Boolean, VumiMessage)
from vumi.tests.helpers import VumiTestCase


class TestBaseField(VumiTestCase):
    def test_validate(self):
        f = Field()
        f.validate("foo")
        f.validate(object())
        self.assertRaises(ValidationError, f.validate, None)

    def test_validate_null(self):
        f = Field(null=True)
        f.validate("foo")
        f.validate(None)

    def test_to_riak(self):
        f = Field()
        obj = object()
        self.assertEqual(f.to_riak(obj), obj)

    def test_from_riak(self):
        f = Field()
        obj = object()
        self.assertEqual(f.from_riak(obj), obj)

    def test_get_descriptor(self):
        f = Field()
        descriptor = f.get_descriptor("foo")
        self.assertEqual(descriptor.key, "foo")
        self.assertEqual(descriptor.field, f)
        self.assertTrue("Field object" in repr(descriptor))


class TestInteger(VumiTestCase):
    def test_validate_unbounded(self):
        i = Integer()
        i.validate(5)
        i.validate(-3)
        self.assertRaises(ValidationError, i.validate, 5.0)
        self.assertRaises(ValidationError, i.validate, "5")

    def test_validate_minimum(self):
        i = Integer(min=3)
        i.validate(3)
        i.validate(4)
        self.assertRaises(ValidationError, i.validate, 2)

    def test_validate_maximum(self):
        i = Integer(max=5)
        i.validate(5)
        i.validate(4)
        self.assertRaises(ValidationError, i.validate, 6)


class TestBoolean(VumiTestCase):

    def test_validate(self):
        b = Boolean()
        b.validate(True)
        b.validate(False)
        self.assertRaises(ValidationError, b.validate, 'True')
        self.assertRaises(ValidationError, b.validate, 'False')
        self.assertRaises(ValidationError, b.validate, 1)
        self.assertRaises(ValidationError, b.validate, 0)


class TestUnicode(VumiTestCase):
    def test_validate(self):
        u = Unicode()
        u.validate(u"")
        u.validate(u"a")
        u.validate(u"æ")
        u.validate(u"foé")
        self.assertRaises(ValidationError, u.validate, "")
        self.assertRaises(ValidationError, u.validate, "foo")
        self.assertRaises(ValidationError, u.validate, 3)

    def test_validate_max_length(self):
        u = Unicode(max_length=5)
        u.validate(u"12345")
        u.validate(u"1234")
        self.assertRaises(ValidationError, u.validate, u"123456")


class TestTag(VumiTestCase):
    def test_validate(self):
        t = Tag()
        t.validate(("pool", "tagname"))
        self.assertRaises(ValidationError, t.validate, ["pool", "tagname"])
        self.assertRaises(ValidationError, t.validate, ("pool",))

    def test_to_riak(self):
        t = Tag()
        self.assertEqual(t.to_riak(("pool", "tagname")), ["pool", "tagname"])

    def test_from_riak(self):
        t = Tag()
        self.assertEqual(t.from_riak(["pool", "tagname"]), ("pool", "tagname"))


class TestTimestamp(VumiTestCase):
    def test_validate(self):
        t = Timestamp()
        t.validate(datetime.now())
        t.validate("2007-01-25T12:00:00Z")
        t.validate(u"2007-01-25T12:00:00Z")
        self.assertRaises(ValidationError, t.validate, "foo")

    def test_to_riak(self):
        t = Timestamp()
        dt = datetime(2100, 10, 5, 11, 10, 9)
        self.assertEqual(t.to_riak(dt), "2100-10-05 11:10:09.000000")

    def test_from_riak(self):
        t = Timestamp()
        dt = datetime(2100, 10, 5, 11, 10, 9)
        self.assertEqual(t.from_riak("2100-10-05 11:10:09.000000"), dt)


class TestJson(VumiTestCase):
    def test_validate(self):
        j = Json()
        j.validate({"foo": None})
        self.assertRaises(ValidationError, j.validate, None)

    def test_to_riak(self):
        j = Json()
        d = {"foo": 5}
        self.assertEqual(j.to_riak(d), d)

    def test_from_riak(self):
        j = Json()
        d = {"foo": [1, 2, 3]}
        self.assertEqual(j.from_riak(d), d)


class TestFieldWithSubtype(VumiTestCase):
    def test_fails_on_fancy_subtype(self):
        self.assertRaises(RuntimeError, FieldWithSubtype, Dynamic())


class TestDynamic(VumiTestCase):
    def test_validate(self):
        dynamic = Dynamic()
        dynamic.validate({u'a': u'foo', u'b': u'bar'})
        self.assertRaises(ValidationError, dynamic.validate,
                          {u'a': 'foo', u'b': u'bar'})
        self.assertRaises(ValidationError, dynamic.validate,
                          u'this is not a dict')
        self.assertRaises(ValidationError, dynamic.validate,
                          {u'a': 'foo', u'b': 2})


class TestListOf(VumiTestCase):
    def test_validate(self):
        listof = ListOf()
        listof.validate([u'foo', u'bar'])
        self.assertRaises(ValidationError, listof.validate,
                          u'this is not a list')
        self.assertRaises(ValidationError, listof.validate, ['a', 2])


class TestSetOf(VumiTestCase):
    def test_validate(self):
        f = SetOf()
        f.validate(set([u'foo', u'bar']))
        self.assertRaises(ValidationError, f.validate, u'this is not a set')
        self.assertRaises(ValidationError, f.validate, set(['a', 2]))

    def test_to_riak(self):
        """
        The JSON representation of a SetOf field is a sorted list.
        """
        f = SetOf()
        self.assertEqual(f.to_riak(set([1, 2, 3])), [1, 2, 3])

    def test_from_riak(self):
        """
        The JSON list is turned into a set when read.
        """
        f = SetOf()
        self.assertEqual(f.from_riak([1, 2, 3]), set([1, 2, 3]))


class TestVumiMessage(VumiTestCase):
    def test_validate(self):
        f = VumiMessage(Message)
        msg = Message()
        f.validate(msg)
        self.assertRaises(
            ValidationError, f.validate, u'this is not a vumi message')
        self.assertRaises(
            ValidationError, f.validate, None)



"""Tests for vumi.middleware.provider_setter."""

from vumi.middleware.provider_setter import (
    StaticProviderSettingMiddleware, AddressPrefixProviderSettingMiddleware,
    ProviderSettingMiddlewareError)
from vumi.tests.helpers import VumiTestCase, MessageHelper


class TestStaticProviderSettingMiddleware(VumiTestCase):
    def setUp(self):
        self.msg_helper = self.add_helper(MessageHelper())

    def mk_middleware(self, config):
        dummy_worker = object()
        mw = StaticProviderSettingMiddleware(
            "static_provider_setter", config, dummy_worker)
        mw.setup_middleware()
        return mw

    def test_set_provider_on_inbound_if_unset(self):
        """
        The statically configured provider value is set on inbound messages
        that have no provider.
        """
        mw = self.mk_middleware({"provider": "MY-MNO"})
        msg = self.msg_helper.make_inbound(None)
        self.assertEqual(msg.get("provider"), None)
        processed_msg = mw.handle_inbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), "MY-MNO")

    def test_replace_provider_on_inbound_if_set(self):
        """
        The statically configured provider value replaces any existing provider
        a message may already have set.
        """
        mw = self.mk_middleware({"provider": "MY-MNO"})
        msg = self.msg_helper.make_inbound(None, provider="YOUR-MNO")
        self.assertEqual(msg.get("provider"), "YOUR-MNO")
        processed_msg = mw.handle_inbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), "MY-MNO")

    def test_set_provider_on_outbound_if_unset(self):
        """
        Outbound messages are left as they are.
        """
        mw = self.mk_middleware({"provider": "MY-MNO"})
        msg = self.msg_helper.make_outbound(None)
        self.assertEqual(msg.get("provider"), None)
        processed_msg = mw.handle_outbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), "MY-MNO")


class TestAddressPrefixProviderSettingMiddleware(VumiTestCase):
    def setUp(self):
        self.msg_helper = self.add_helper(MessageHelper())

    def mk_middleware(self, config):
        dummy_worker = object()
        mw = AddressPrefixProviderSettingMiddleware(
            "address_prefix_provider_setter", config, dummy_worker)
        mw.setup_middleware()
        return mw

    def assert_middleware_error(self, msg):
        [err] = self.flushLoggedErrors(ProviderSettingMiddlewareError)
        self.assertEqual(str(err.value), msg)

    def test_set_provider_unique_matching_prefix(self):
        """
        If exactly one prefix matches the address, its corresponding provider
        value is set on the inbound message.
        """
        mw = self.mk_middleware({"provider_prefixes": {
            "+123": "MY-MNO",
            "+124": "YOUR-MNO",
        }})
        msg = self.msg_helper.make_inbound(None, from_addr="+12345")
        self.assertEqual(msg.get("provider"), None)
        processed_msg = mw.handle_inbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), "MY-MNO")

    def test_set_provider_longest_matching_prefix(self):
        """
        If more than one prefix matches the address, the provider value for
        the longest matching prefix is set on the inbound message.
        """
        mw = self.mk_middleware({"provider_prefixes": {
            "+12": "YOUR-MNO",
            "+123": "YOUR-MNO",
            "+1234": "YOUR-MNO",
            "+12345": "MY-MNO",
            "+123456": "YOUR-MNO",
        }})
        msg = self.msg_helper.make_inbound(None, from_addr="+12345")
        self.assertEqual(msg.get("provider"), None)
        processed_msg = mw.handle_inbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), "MY-MNO")

    def test_no_provider_for_no_matching_prefix(self):
        """
        If no prefix matches the address, the provider value will be set to
        ``None`` on the inbound message.
        """
        mw = self.mk_middleware({"provider_prefixes": {
            "+124": "YOUR-MNO",
            "+125": "YOUR-MNO",
        }})
        msg = self.msg_helper.make_inbound(None, from_addr="+12345")
        self.assertEqual(msg.get("provider"), None)
        processed_msg = mw.handle_inbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), None)

    def test_set_provider_no_normalize_msisdn(self):
        """
        If exactly one prefix matches the address, its corresponding provider
        value is set on the inbound message.
        """
        mw = self.mk_middleware({
            "provider_prefixes": {
                "083": "MY-MNO",
                "+2783": "YOUR-MNO",
            },
        })
        msg = self.msg_helper.make_inbound(None, from_addr="0831234567")
        self.assertEqual(msg.get("provider"), None)
        processed_msg = mw.handle_inbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), "MY-MNO")

    def test_set_provider_normalize_msisdn(self):
        """
        If exactly one prefix matches the address, its corresponding provider
        value is set on the inbound message.
        """
        mw = self.mk_middleware({
            "normalize_msisdn": {"country_code": "27"},
            "provider_prefixes": {
                "083": "YOUR-MNO",
                "+2783": "MY-MNO",
            },
        })
        msg = self.msg_helper.make_inbound(None, from_addr="0831234567")
        self.assertEqual(msg.get("provider"), None)
        processed_msg = mw.handle_inbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), "MY-MNO")

    def test_set_provider_normalize_msisdn_strip_plus(self):
        """
        If exactly one prefix matches the address, its corresponding provider
        value is set on the inbound message.
        """
        mw = self.mk_middleware({
            "normalize_msisdn": {"country_code": "27", "strip_plus": True},
            "provider_prefixes": {
                "083": "YOUR-MNO",
                "+2783": "YOUR-MNO",
                "2783": "MY-MNO",
            },
        })
        msg = self.msg_helper.make_inbound(None, from_addr="0831234567")
        self.assertEqual(msg.get("provider"), None)
        processed_msg = mw.handle_inbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), "MY-MNO")

    def test_set_provider_on_outbound(self):
        """
        Outbound messages are left as they are.
        """
        mw = self.mk_middleware({"provider_prefixes": {"+123": "MY-MNO"}})
        msg = self.msg_helper.make_outbound(
            None, to_addr="+1234567", from_addr="+12345")
        self.assertEqual(msg.get("provider"), None)
        processed_msg = mw.handle_outbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), "MY-MNO")

    def test_provider_not_overwritten_for_inbound(self):
        """
        If a provider already exists for an inbound message, it isn't
        overwritten.
        """
        mw = self.mk_middleware({"provider_prefixes": {"+123": "MY-MNO"}})
        msg = self.msg_helper.make_inbound(
            None, to_addr="+345", from_addr="+12345", provider="OTHER-MNO")
        processed_msg = mw.handle_inbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), "OTHER-MNO")

    def test_provider_not_overwritten_for_outbound(self):
        """
        If a provider already exists for an outbound message, it isn't
        overwritten.
        """
        mw = self.mk_middleware({"provider_prefixes": {"+123": "MY-MNO"}})
        msg = self.msg_helper.make_outbound(
            None, to_addr="+1234567", from_addr="+345", provider="OTHER-MNO")
        processed_msg = mw.handle_outbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), "OTHER-MNO")

    def test_provider_logs_no_address_error_for_inbound(self):
        """
        If the from_addr of an inbound message is None, an error should be
        logged and the message returned.
        """
        mw = self.mk_middleware({"provider_prefixes": {"+123": "MY-MNO"}})
        msg = self.msg_helper.make_inbound(
            None, to_addr="+1234567", from_addr=None)
        processed_msg = mw.handle_inbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), None)
        self.assert_middleware_error(
            "Address for determining message provider cannot be None,"
            " skipping message")

    def test_provider_logs_no_address_error_for_outbound(self):
        """
        If the to_addr of an outbound message is None, an error should be
        logged and the message returned.
        """
        mw = self.mk_middleware({"provider_prefixes": {"+123": "MY-MNO"}})
        msg = self.msg_helper.make_outbound(
            None, to_addr=None, from_addr="+345")
        processed_msg = mw.handle_outbound(msg, "dummy_connector")
        self.assertEqual(processed_msg.get("provider"), None)
        self.assert_middleware_error(
            "Address for determining message provider cannot be None,"
            " skipping message")
